"""
Created on 2 Aug 2016

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdCSVWriter(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [{ -a | -x | -s }] [-q] [-e] [-v] [FILENAME]",
                                              version=version())

        # mode...
        self.__parser.add_option("--append", "-a", action="store_true", dest="append", default=False,
                                 help="append rows to existing file")

        self.__parser.add_option("--exclude-header", "-x", action="store_true", dest="exclude_header", default=False,
                                 help="do not write the header row to stdout")

        self.__parser.add_option("--header-scan", "-s", action="store_true", dest="header_scan", default=False,
                                 help="scan all documents before building the header row")

        # output...
        self.__parser.add_option("--quote-all", "-q", action="store_true", dest="quote_all", default=False,
                                 help="wrap all CSV cell values in quotes")

        self.__parser.add_option("--echo", "-e", action="store_true", dest="echo", default=False,
                                 help="echo stdin to stdout")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.append:
            count += 1

        if self.exclude_header:
            count += 1

        if self.header_scan:
            count += 1

        if count > 1:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def append(self):
        return self.__opts.append


    @property
    def exclude_header(self):
        return self.__opts.exclude_header


    @property
    def header_scan(self):
        return self.__opts.header_scan


    @property
    def quote_all(self):
        return self.__opts.quote_all


    @property
    def echo(self):
        return self.__opts.echo


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def filename(self):
        return self.__args[0] if len(self.__args) > 0 else None


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdCSVWriter:{append:%s, exclude_header:%s, header_scan:%s, quote_all:%s, echo:%s, " \
               "verbose:%s, filename:%s}" % \
                    (self.append, self.exclude_header, self.header_scan, self.quote_all, self.echo,
                     self.verbose, self.filename)



#!/usr/bin/env python3

"""
Created on 8 Aug 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_localize utility is used to shift the rec datetime field of each input document from its current offset to
the offset determined by the given timezone.

When the transform is applied, the the date / time is shifted, and the UTC offset field is altered accordingly. Thus,
the resulting localised datetime represents the same point in global history, presented for a alternative UTC offset.

In most cases, the UTC offset of a timezone is dependent on daylight saving time. The sample_localize utility is
sensitive to this - the adjustment to the UTC offset of each document's rec field is dependent on its specific
date / time value. Thus, UTC offsets are not the same thing as timezones! It is always safe to shift from UTC to a
timezone, but sifting to a subsequent timezone can have unexpected results, particularly on daylight saving time
boundaries.

The sample_localize utility halts on input lines which have no datetime field, or a datetime field with a
malformed ISO 8601 datetime.

Note that the timezone of a South Coast Science device is normally reported on its status topic.

SYNOPSIS
sample_localize.py { -z | -t TIMEZONE_NAME [-i ISO_PATH] [-v]

EXAMPLES
aws_topic_history.py -s 2023-03-26T00:50:00Z -e 2023-03-26T01:10:00Z \
south-coast-science-demo/brighton-urban/loc/1/climate | \
sample_localize.py -v -t Europe/London

DOCUMENT EXAMPLE - INPUT
{"val": {"hmd": 73.6, "tmp": 12.0, "bar": {"pA": 99.272}}, "rec": "2023-03-26T01:00:09Z",
"ver": 1.0, "tag": "scs-bgx-570"}

DOCUMENT EXAMPLE - OUTPUT
{"val": {"hmd": 73.6, "tmp": 12.0, "bar": {"pA": 99.272}}, "rec": "2023-03-26T02:00:09+01:00",
"ver": 1.0, "tag": "scs-bgx-570"}

SEE ALSO
scs_analysis/sample_iso_8601
scs_analysis/sample_time_shift

RESOURCES
https://en.wikipedia.org/wiki/ISO_8601
https://github.com/south-coast-science/scs_dev/wiki/3:-Data-formats
http://pytz.sourceforge.net
"""

import sys

from scs_analysis.cmd.cmd_sample_localize import CmdSampleLocalize

from scs_core.data.datetime import LocalizedDatetime
from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict

from scs_core.location.timezone import Timezone

from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleLocalize()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('sample_localize', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # helper...

        if cmd.zones:
            for zone in Timezone.zones():
                print(zone, file=sys.stderr)
            exit(0)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        if not Timezone.is_valid(cmd.timezone):
            logger.error("unrecognised timezone: '%s'." % cmd.timezone)
            exit(2)

        timezone = Timezone(cmd.timezone)
        zone = timezone.zone


        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            jstr = line.strip()
            datum = PathDict.construct_from_jstr(jstr)

            if datum is None:
                continue

            document_count += 1

            try:
                iso_node = datum.node(cmd.iso)
            except KeyError:
                logger.error("ISO path '%s' not in %s" % (cmd.iso, jstr))
                exit(1)

            iso = LocalizedDatetime.construct_from_iso8601(iso_node)

            if iso is None:
                logger.error("malformed datetime '%s' in %s" % (iso_node, jstr))
                exit(1)

            datum.append(cmd.iso, iso.localize(zone).as_iso8601())

            # report...
            print(JSONify.dumps(datum))
            sys.stdout.flush()

            processed_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    finally:
        if not cmd.zones:
            logger.info("documents: %d processed: %d" % (document_count, processed_count))



#!/usr/bin/env python3

"""
Created on 15 Feb 2019

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_iso_8601 utility is used to replace non-localised datetime fields with an ISO 8601 localised datetime field
for JSON documents of any schema.

Dates may be in the format:

* DD-MM-YYYY
* DD/MM/YYYY
* DD/MM/YY
* DD_MMM_YYYY
* MM-DD-YYYY
* MM/DD/YYYY
* MM/DD/YY
* YYYY-MM-DD
* YYYY/MM/DD
* OLE Automation date

Times in the 24-hour format HH:MM or HH:MM:SS. For datetime fields, the
format may be YYYY-MM-DD HH:MM or YYYY-MM-DD HH:MM:SS. Hour values may exceed the range 0-23. If fields are missing
from the input document or are malformed, execution will terminate.

The --no-time flag should be used where the input document contains a date but no time indicator. In this case, the
the data is interpreted as being an aggregate of the whole date period. For example, if the input date is 31/01/2022
then the output datetime will be 2022-02-01T00:00:00 (the date is moved on by one day because the datetime field
indicates the end of the sample period).

If the input datetimes are not UTC, then the timezone of the input data should be specified. In this case, the
datetime may optionally be shifted to UTC.

All fields in the input document are presented in the output document, with the exception of the selected date, time or
datetime fields. The default name for the ISO 8601 datetime output field is 'rec' but an alternate name may be
specified.

SYNOPSIS
sample_iso_8601.py { -z | { -o | -f DATE_FORMAT } [-t TIMEZONE_NAME [-u]] [-i ISO_PATH]
{ DATETIME_PATH [-n] | DATE_PATH TIME_PATH } } [-s] [-v]

EXAMPLES
csv_reader.py 15_min_Praxis_LHR2.csv -l10 | sample_iso_8601.py -v -f DD/MM/YYYY "Max of Time" -t Europe/Athens -u

DOCUMENT EXAMPLE - INPUT
{"Max of Time": "08/02/2019 00:00", "Average of praxis-431": {"val": {"NO2": {"cnc": 24}, "NO": {"cnc": 67.54666667}}},
"Average of ref": {"NOCNC1 (Processed)": 1.606666667, "NO2CNC1 (Processed)": 6.473333333},
"15 minute \"real\" data": 5.92, "rec": "2019-02-08T00:00:00Z"}

DOCUMENT EXAMPLE - OUTPUT
{"rec": "2019-02-07T22:00:00Z", "Average of praxis-431": {"val": {"NO2": {"cnc": 24}, "NO": {"cnc": 67.54666667}}},
"Average of ref": {"NOCNC1 (Processed)": 1.606666667, "NO2CNC1 (Processed)": 6.473333333},
"15 minute \"real\" data": 5.92}

SEE ALSO
scs_analysis/sample_localize
scs_analysis/sample_time_shift

RESOURCES
https://en.wikipedia.org/wiki/ISO_8601
https://github.com/south-coast-science/scs_dev/wiki/3:-Data-formats
https://docs.microsoft.com/en-us/dotnet/api/system.datetime.tooadate?view=netframework-4.8
"""

import sys

from scs_analysis.cmd.cmd_sample_iso_8601 import CmdSampleISO8601

from scs_core.data.datetime import DateParser, LocalizedDatetime
from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict

from scs_core.location.timezone import Timezone

from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    parser = None
    timezone = None
    zone = None
    pieces = None

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleISO8601()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('sample_iso_8601', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # validation...

        if not cmd.zones and not cmd.oad and not DateParser.is_valid_format(cmd.format):
            logger.error("unsupported format: %s" % cmd.format)
            exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        # format...
        if cmd.format:
            parser = DateParser.construct(cmd.format)
            logger.info(parser)

        # timezone...
        if cmd.timezone is not None:
            try:
                timezone = Timezone(cmd.timezone)
                zone = timezone.zone

            except ValueError:
                logger.error("unrecognised timezone: '%s'." % cmd.timezone)
                exit(2)

            logger.info(timezone)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.zones:
            for zone in Timezone.zones():
                print(zone, file=sys.stderr)
            exit(0)

        for line in sys.stdin:
            jstr = line.strip()
            datum = PathDict.construct_from_jstr(jstr)

            if datum is None:
                continue

            document_count += 1

            paths = datum.paths()

            if cmd.oad:
                # OAD...
                if cmd.datetime_path not in paths:
                    logger.error("datetime path '%s' not in %s" % (cmd.datetime_path, jstr))
                    exit(1)

                # ISO 8601...
                iso = LocalizedDatetime.construct_from_oad(datum.node(cmd.datetime_path), tz=zone)

            elif cmd.uses_datetime():
                # datetime...
                if cmd.datetime_path not in paths:
                    logger.error("datetime path '%s' not in %s" % (cmd.datetime_path, jstr))
                    exit(1)

                try:
                    pieces = datum.node(cmd.datetime_path).rsplit(' ', 1)           # split on last space character

                except AttributeError:
                    if cmd.skip_malformed:
                        continue

                    logger.error("malformed datetime (AttributeError) '%s' in %s" % (cmd.datetime_path, jstr))
                    exit(1)

                if (cmd.no_time and len(pieces) != 1) or (not cmd.no_time and len(pieces) != 2):
                    if cmd.skip_malformed:
                        continue

                    logger.error("malformed datetime (field count) '%s' in %s" % (cmd.datetime_path, jstr))
                    exit(1)

                date = pieces[0].strip()
                time = '24:00:00' if cmd.no_time else pieces[1].strip()

                # ISO 8601...
                iso = LocalizedDatetime.construct_from_date_time(parser, date, time, tz=zone)

            else:
                # date / time...
                if cmd.date_path not in paths:
                    logger.error("date path '%s' not in %s" % (cmd.date_path, jstr))
                    exit(1)

                if cmd.time_path not in paths:
                    logger.error("time path '%s' not in %s" % (cmd.time_path, jstr))
                    exit(1)

                date = datum.node(cmd.date_path)
                time = datum.node(cmd.time_path)

                # ISO 8601...
                iso = LocalizedDatetime.construct_from_date_time(parser, date, time, tz=zone)

            if iso is None:
                if cmd.skip_malformed:
                    continue

                logger.error("malformed date/time in %s" % jstr)
                exit(1)

            if cmd.timezone is not None and cmd.utc:
                iso = iso.utc()

            target = PathDict()
            target.append(cmd.iso, iso.as_iso8601())

            # copy...
            for path in paths:
                if path not in cmd.datetime_paths():
                    target.append(path, datum.node(path))

            # report...
            print(JSONify.dumps(target))
            sys.stdout.flush()

            processed_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except KeyError as ex:
        logger.error(repr(ex))
        exit(1)

    finally:
        if not cmd.zones:
            logger.info("documents: %d processed: %d" % (document_count, processed_count))



"""
Created on 20 Mar 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdSampleMedian(object):
    """
    unix command line handler
    """

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-w SIZE] [-p PRECISION] [-v] [PATH]",
                                              version=version())

        # optional...
        self.__parser.add_option("--window", "-w", type="int", action="store", dest="window", default=3,
                                 help="window size (must be an odd number, default 3)")

        # output...
        self.__parser.add_option("--prec", "-p", type="int", action="store", default=None, dest="precision",
                                 help="precision (default 0 decimal places)")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.window is not None and self.window % 2 == 0:
            return False

        if self.__args and len(self.__args) != 1:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def window(self):
        return self.__opts.window


    @property
    def precision(self):
        return self.__opts.precision


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def path(self):
        return self.__args[0] if len(self.__args) > 0 else None


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdSampleMedian:{window:%s, verbose:%s, precision:%s, path:%s}" % \
               (self.window, self.verbose, self.precision, self.path)



#!/usr/bin/env python3

"""
Created on 24 Mar 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_low_pass utility provides a rolling exponential average or low pass filter. The user must specify the
sampling interval in seconds, together with the cut-off frequency.

Input data is typically in the form of a JSON document. A command parameter specifies the path to the node within
the document that is to be filtered. The node is typically a leaf node integer or float. The output of the
sample_low_pass utility includes the source value, and the smoothed value.

SYNOPSIS
sample_low_pass.py -d DELTA_T -c CUT_OFF [-p PRECISION] [-v] [PATH]

EXAMPLES
aws_topic_history.py -m60 /orgs/south-coast-science-demo/brighton/loc/1/gases | \
sample_low_pass.py -d 10.0 -c 0.02 -p 1 val.CO.cnc

DOCUMENT EXAMPLE - INPUT
{"tag": "scs-bgx-401", "rec": "2018-03-27T09:54:41.042+00:00", "val": {
"NO2": {"weV": 0.29563, "aeV": 0.280879, "weC": 0.009569, "cnc": 61.0},
"Ox": {"weV": 0.406819, "aeV": 0.387443, "weC": -0.010706, "cnc": 34.7},
"NO": {"weV": 0.319692, "aeV": 0.292129, "weC": 0.028952, "cnc": 165.5},
"CO": {"weV": 0.395819, "aeV": 0.289317, "weC": 0.113108, "cnc": 311.3},
"sht": {"hmd": 82.4, "tmp": 12.6}}}

DOCUMENT EXAMPLE - OUTPUT
{"rec": "2018-03-27T10:55:11.033+00:00", "val": {"CO": {"cnc": {"src": 121.3, "lpf": 131.3}}}}

RESOURCES
Ten Little Algorithms, Part 2: The Single-Pole Low-Pass Filter
https://www.embeddedrelated.com/showarticle/779.php
"""

import sys

from scs_analysis.cmd.cmd_sample_low_pass import CmdLowPass

from scs_core.data.json import JSONify
from scs_core.data.low_pass_filter import LowPassFilter
from scs_core.data.path_dict import PathDict


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdLowPass()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    if cmd.verbose:
        print("sample_low_pass: %s" % cmd, file=sys.stderr)


    # ----------------------------------------------------------------------------------------------------------------
    # resources...

    lpf = LowPassFilter.construct(cmd.delta, cmd.cut_off)

    if cmd.verbose:
        print("sample_low_pass: %s" % lpf, file=sys.stderr)
        sys.stderr.flush()

    try:
        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            datum = PathDict.construct_from_jstr(line)

            if datum is None:
                break

            document_count += 1

            value = datum.node(cmd.path)

            if value is None:
                break

            target = PathDict()

            if datum.has_path('rec'):
                target.copy(datum, 'rec')

            target.append(cmd.path + '.src', value)
            target.append(cmd.path + '.lpf', round(lpf.line(value), cmd.precision))

            print(JSONify.dumps(target.node()))
            sys.stdout.flush()

            processed_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except KeyError as ex:
        print("sample_low_pass: %s" % repr(ex), file=sys.stderr)
        exit(1)

    finally:
        if cmd.verbose:
            print("sample_low_pass: documents: %d processed: %d" % (document_count, processed_count), file=sys.stderr)



#!/usr/bin/env python3

"""
Created on 24 Nov 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The cognito_user_identity utility is used to create, update and retrieve the AWS Cognito identity for the user.

If the --Create function is used, an email is sent to the new user. The verification link in the email must be
excercised in order for the account to gain a CONFIRMED status.

SYNOPSIS
cognito_user_identity.py [-c CREDENTIALS] | -C | -R | -U } [-i INDENT] [-v]

EXAMPLES
./cognito_user_identity.py -R

DOCUMENT EXAMPLE
{"username": "506cd055-1978-4984-9f17-2fad77797fa1", "email": "bruno.beloff@southcoastscience.com",
"given-name": "Bruno", "family-name": "Beloff", "confirmation-status": "CONFIRMED", "enabled": true,
"email-verified": true, "is-super": true, "is-tester": true, "is-financial": true,
"created": "2023-04-20T11:45:21Z", "last-updated": "2023-06-26T14:39:17Z"}

SEE ALSO
scs_analysis/cognito_user_credentials

RESOURCES
https://docs.aws.amazon.com/cognito/latest/developerguide/signing-up-users-in-your-app.html
https://docs.aws.amazon.com/cognito/latest/developerguide/user-pool-settings-policies.html
"""

import sys

from scs_analysis.cmd.cmd_cognito_user_identity import CmdCognitoUserIdentity

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager
from scs_core.aws.security.cognito_user import CognitoUserIdentity
from scs_core.aws.security.cognito_user_finder import CognitoUserFinder
from scs_core.aws.security.cognito_user_manager import CognitoUserCreator, CognitoUserEditor

from scs_core.client.http_exception import HTTPException, HTTPConflictException

from scs_core.data.datum import Datum
from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.comms.stdio import StdIO
from scs_host.sys.host import Host


# TODO: check for newlines on interactivity
# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    logger = None
    gatekeeper = None
    credentials = None
    auth = None
    finder = None
    report = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdCognitoUserIdentity()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('cognito_user_identity', verbose=cmd.verbose)
        logger = Logging.getLogger()

        logger.info(cmd)


        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        if not cmd.create:
            credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

            if not credentials:
                exit(1)

            gatekeeper = CognitoLoginManager()
            auth = gatekeeper.user_login(credentials)

            if not auth.is_ok():
                logger.error("login: %s." % auth.authentication_status.description)
                exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        if not cmd.create:
            finder = CognitoUserFinder()


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.retrieve:
            report = finder.get_self(auth.id_token)

        if cmd.create:
            # create...
            given_name = StdIO.prompt("Enter given name")
            family_name = StdIO.prompt("Enter family name")
            email = StdIO.prompt("Enter email address")
            password = StdIO.prompt("Enter password")
            retrieval_password = StdIO.prompt("Enter retrieval password (RETURN for same)")

            if not retrieval_password:
                retrieval_password = password

            # validation...
            if not given_name or not given_name:
                logger.error("Given name and family name are required.")
                exit(1)

            if not Datum.is_email_address(email):
                logger.error("The email address is not valid.")
                exit(1)

            if not CognitoUserIdentity.is_valid_password(password):
                logger.error("The password must include lower and upper case, numeric and punctuation characters.")
                exit(1)

            # save identity...
            identity = CognitoUserIdentity(None, None, None, True, False, email,
                                           given_name, family_name, password, False, False, False, None)

            manager = CognitoUserCreator()
            report = manager.create(identity)

            # create credentials...
            credentials = CognitoClientCredentials(cmd.credentials_name, email, password, retrieval_password)
            credentials.save(Host, encryption_key=retrieval_password)

        if cmd.update:
            # find...
            identity = finder.get_self(auth.id_token)

            # update identity...
            given_name = StdIO.prompt("Enter given name", default=identity.given_name)
            family_name = StdIO.prompt("Enter family name", default=identity.family_name)
            email = StdIO.prompt("Enter email address", default=identity.email)
            password = StdIO.prompt("Enter password (RETURN to keep existing)")

            if not password:
                password = credentials.password

            if credentials.retrieval_password == credentials.password:
                retrieval_password = StdIO.prompt("Enter retrieval password (RETURN for same)")

                if not retrieval_password:
                    retrieval_password = password

            else:
                retrieval_password = StdIO.prompt("Enter retrieval password (RETURN to keep existing)")

                if not retrieval_password:
                    retrieval_password = credentials.retrieval_password

            # validation...
            if not given_name or not given_name:
                logger.error("Given name and family name are required.")
                exit(1)

            if not Datum.is_email_address(email):
                logger.error("The email address '%s' is not valid." % email)
                exit(1)

            if password and not CognitoUserIdentity.is_valid_password(password):
                logger.error("The password '%s' is not valid." % password)
                exit(1)

            # save identity...
            identity = CognitoUserIdentity(identity.username, None, None, True, identity.email_verified, email,
                                           given_name, family_name, password, identity.is_super, identity.is_tester,
                                           identity.is_financial, None)

            auth = gatekeeper.user_login(credentials)                          # renew credentials
            manager = CognitoUserEditor()
            report = manager.update(auth.id_token, identity)

            # update credentials...
            credentials = CognitoClientCredentials(credentials.name, email, password, retrieval_password)
            credentials.save(Host, encryption_key=retrieval_password)


    # ----------------------------------------------------------------------------------------------------------------
    # end...

        if report is not None:
            print(JSONify.dumps(report, indent=cmd.indent))

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPConflictException as ex:
        logger.error("the email address '%s' is already in use." % report.email)
        exit(1)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



#!/usr/bin/env python3

"""
Created on 18 Jan 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The organisation_user_paths utility is used to

SYNOPSIS
organisation_user_paths.py  [-c CREDENTIALS] { -F -e EMAIL -r PATH_ROOT | \
-C -e EMAIL -r PATH_ROOT -x PATH_EXTENSION | \
-D -e EMAIL -r PATH_ROOT -x PATH_EXTENSION } \
[-i INDENT] [-v]

EXAMPLES
organisation_user_paths.py -F -u NARA

DOCUMENT EXAMPLE
{"Username": 111, "OrgID": 1, "IsOrgAdmin": true, "IsDeviceAdmin": true, "IsSuspended": false}

SEE ALSO
scs_analysis/cognito_user_credentials
"""

import sys

from scs_analysis.cmd.cmd_organisation_user_paths import CmdOrganisationUserPaths

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager

from scs_core.aws.security.organisation import OrganisationPathRoot, OrganisationUserPath
from scs_core.aws.security.organisation_manager import OrganisationManager
from scs_core.aws.security.cognito_user_finder import CognitoUserFinder

from scs_core.data.datum import Datum
from scs_core.data.json import JSONify

from scs_core.client.http_exception import HTTPException

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    logger = None
    report = []

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdOrganisationUserPaths()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('organisation_user_paths', verbose=cmd.verbose)
        logger = Logging.getLogger()

        logger.info(cmd)

        if not Datum.is_email_address(cmd.email):
            logger.error("email address '%s' is not valid." % cmd.email)
            exit(2)

        if cmd.path_root and not OrganisationPathRoot.is_valid_path_root(cmd.path_root):
            logger.error("path root '%s' is not valid." % cmd.path_root)
            exit(2)

        if cmd.path_extension is not None and not OrganisationUserPath.is_valid_path_extension(cmd.path_extension):
            logger.error("path extension '%s' is not valid." % cmd.path_extension)
            exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

        if not credentials:
            exit(1)

        gatekeeper = CognitoLoginManager()
        auth = gatekeeper.user_login(credentials)

        if not auth.is_ok():
            logger.error("login: %s." % auth.authentication_status.description)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        finder = CognitoUserFinder()
        manager = OrganisationManager()


        # ------------------------------------------------------------------------------------------------------------
        # validation...

        cognito = finder.get_by_email(auth.id_token, cmd.email)

        if cognito is None:
            logger.error("no Cognito user found for email: '%s'." % cmd.email)
            exit(1)

        if cmd.path_root:
            opr = manager.get_opr_by_path_root(auth.id_token, cmd.path_root)

            if opr is None:
                logger.error("no organisation path root found for path: '%s'." % cmd.path_root)
                exit(1)

            opr_id = opr.opr_id

        else:
            opr_id = None


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.find:
            report = manager.find_oups(auth.id_token, username=cognito.username, opr_id=opr_id)

        if cmd.create:
            report = OrganisationUserPath(cognito.username, opr_id, cmd.path_extension)
            manager.assert_oup(auth.id_token, report)

        if cmd.delete:
            oup = OrganisationUserPath(cognito.username, opr_id, cmd.path_extension)
            manager.delete_oup(auth.id_token, oup)


    # ----------------------------------------------------------------------------------------------------------------
    # end...

        if not cmd.delete:
            print(JSONify.dumps(report, indent=cmd.indent))

        if cmd.find:
            logger.info("found: %s" % len(report))

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



#!/usr/bin/env python3

"""
Created on 26 Feb 2019

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_subset utility is used to find a subset of documents whose value for a specified field lies either
inside or outside one or two bounding values.

Input is in the form of a stream of JSON documents. Documents are written to stdout if they match the specification,
and discarded otherwise. Documents which do not have the specified field, or have an empty field value are also
discarded. If a field value is present but cannot be cast to the correct type, then the sample_subset utility
terminates.

The type of the field may be specified explicitly as either numeric or ISO 8601 datetime. If no specification is given,
the value is interpreted as a string.

Evaluation follows the rule:

lower bound <= value < upper bound

Both upper and lower bounds are optional. If both are present, then the lower bound value must be less than the upper
bound value. If neither are present, then the sample_subset utility filters out documents with missing fields or
empty values. An alternative test is --equal.

If the --exclusions flag is used, the sample_subset utility outputs only the documents that do not fit within the
specification. Note that, in this case, documents with missing or empty fields are still discarded.

SYNOPSIS
sample_subset.py { -i | -n  | -s } { [-e EQUAL] | [-l LOWER] [-u UPPER] } [-s] [-x] [-v] PATH

EXAMPLES
csv_reader.py praxis_303.csv | \
sample_subset.py -v -i -l 2018-09-26T00:00:00Z -u 2018-09-27T00:00:00Z rec

csv_reader.py -v scs-bgx-431-ref-meteo-gases-2020-H1-slp.csv | \
sample_subset.py -v "ref.NO2 Processed Measurement (ppb)" | \
csv_writer.py -v scs-bgx-431-ref-meteo-gases-2020-H1-slp-no2.csv
"""

import sys

from scs_analysis.cmd.cmd_sample_subset import CmdSampleSubset

from scs_core.data.datetime import LocalizedDatetime
from scs_core.data.datum import Datum
from scs_core.data.path_dict import PathDict

from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    equal_bound = None
    lower_bound = None
    upper_bound = None
    value = None

    document_count = 0
    processed_count = 0
    output_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleSubset()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('sample_subset (%s)' % cmd.path, verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # resources...

        try:
            equal_bound = cmd.equal
        except ValueError as ex:
            logger.error("invalid value for equal: %s" % ex)
            exit(2)

        try:
            lower_bound = cmd.lower
        except ValueError as ex:
            logger.error("invalid value for lower bound: %s" % ex)
            exit(2)

        try:
            upper_bound = cmd.upper
        except ValueError as ex:
            logger.error("invalid value for upper bound: %s" % ex)
            exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            is_member = True

            jstr = line.strip()
            datum = PathDict.construct_from_jstr(jstr)

            if datum is None:
                continue

            document_count += 1

            paths = datum.paths()

            # value...
            if cmd.path not in paths:
                continue

            value_node = datum.node(cmd.path)

            if value_node == '':
                continue

            if cmd.iso8601:
                value = LocalizedDatetime.construct_from_iso8601(value_node)

                if value is None:
                    if cmd.strict:
                        logger.error("invalid ISO 8601 value '%s' in %s" % (value_node, jstr))
                        exit(1)
                    else:
                        is_member = False

            elif cmd.numeric:
                value = Datum.float(value_node)

                if value is None:
                    if cmd.strict:
                        logger.error("invalid numeric value '%s' in %s" % (value_node, jstr))
                        exit(1)
                    else:
                        is_member = False

            else:
                value = value_node

                if value is None or value == '':
                    continue

            processed_count += 1

            # bounds...
            if lower_bound is not None or upper_bound is not None:
                try:
                    if equal_bound is not None:
                        is_member = value == equal_bound
                    else:
                        is_member = (lower_bound is None or value >= lower_bound) and \
                                    (upper_bound is None or value < upper_bound)

                except TypeError as ex:
                    if cmd.strict:
                        logger.error("TypeError: %s" % jstr)
                        exit(1)
                    else:
                        continue

            # inclusions...
            if (cmd.inclusions and not is_member) or (cmd.exclusions and is_member):
                continue

            # report...
            print(jstr)
            sys.stdout.flush()

            output_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except KeyError as ex:
        logger.error(repr(ex))
        exit(1)

    finally:
        logger.info("documents: %d processed: %d output: %d" % (document_count, processed_count, output_count))



#!/usr/bin/env python3

"""
Created on 29 Jun 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

DESCRIPTION
The alert_status utility is used to report on the history of alerts generated by a specific alert specification.
Alert causes are:

* L - Below lower threshold
* U - Above upper threshold
* N - Null value
* OK - data has returned within bounds

SYNOPSIS
alert_status.py [-c CREDENTIALS] { -F { -l | -d [-a CAUSE] } | -D } [-i INDENT] [-v] ID

EXAMPLES
alert_status.py -vi4 -c bruno -Fl 87

alert_status.py -v -c bruno -Fd 107 | node.py -s | csv_writer.py

DOCUMENT EXAMPLE
{"id": 88, "rec": "2023-06-29T09:48:42Z", "cause": "NV", "val": null}

SEE ALSO
scs_analysis/alert
scs_analysis/cognito_user_credentials
"""

import sys

from scs_analysis.cmd.cmd_alert_status import CmdAlertStatus

from scs_core.aws.monitor.alert.alert_status_manager import AlertStatusManager

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager

from scs_core.client.http_exception import HTTPException, HTTPNotFoundException

from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    response = None
    report = None

    # ------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdAlertStatus()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('alert_status', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

        if not credentials:
            exit(1)

        gatekeeper = CognitoLoginManager()
        auth = gatekeeper.user_login(credentials)

        if not auth.is_ok():
            logger.error("login: %s." % auth.authentication_status.description)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        status_manager = AlertStatusManager()


        # ------------------------------------------------------------------------------------------------------------
        # validation...

        try:
            int(cmd.id)
        except (TypeError, ValueError):
            logger.error('the ID must be an integer.')
            exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.find:
            if cmd.latest:
                response = status_manager.find(auth.id_token, cmd.id, None, cmd.response_mode())
                report = response.alert_statuses[0] if response.alert_statuses else None

            if cmd.history:
                response = status_manager.find(auth.id_token, cmd.id, cmd.cause, cmd.response_mode())
                report = sorted(response.alert_statuses)

        if cmd.delete:
            status_manager.delete(auth.id_token, cmd.id)


        # ------------------------------------------------------------------------------------------------------------
        # end...

        # report...
        if report is not None:
            print(JSONify.dumps(report, indent=cmd.indent))

        if cmd.history:
            logger.info('retrieved: %s' % len(response.alert_statuses))


    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPNotFoundException:
        logger.error("no alert found with ID %s." % cmd.id)
        exit(2)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



#!/usr/bin/env python3

"""
Created on 28 Jun 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

DESCRIPTION
The device monitor periodically checks on the availability and health of every air quality monitoring device. The
device_monitor utility is used to manage the email addresses associated with individual devices.

SYNOPSIS
device_monitor.py [-c CREDENTIALS] { -F [{ -e EMAIL_ADDR | -t DEVICE_TAG } [-x]] | -A EMAIL_ADDR DEVICE_TAG [-j] |
-S DEVICE_TAG { 0 | 1 } | -D EMAIL_ADDR [{ -t DEVICE_TAG | -f }] } [-i INDENT] [-v]

EXAMPLES
device_monitor.py -c super -Ft scs-opc-109

DOCUMENT EXAMPLE
{"scs-be2-3": {"device-tag": "scs-be2-3", "recipients": [{"email": "bbeloff@me.com", "json-message": true},
{"email": "bruno.beloff@southcoastscience.com", "json-message": false}], "suspended": false}}

SEE ALSO
scs_analysis/cognito_user_credentials
scs_analysis/device_monitor_status
"""

import sys

from scs_analysis.cmd.cmd_device_monitor import CmdDeviceMonitor

from scs_core.aws.manager.byline.byline_finder import BylineFinder

from scs_core.aws.monitor.device.device_monitor_specification_manager import DeviceMonitorSpecificationManager

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_device import CognitoDeviceCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager

from scs_core.client.http_exception import HTTPException, HTTPNotFoundException

from scs_core.data.datum import Datum
from scs_core.data.json import JSONify

from scs_core.email.email import EmailRecipient

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    response = None
    report = None

    # ------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdDeviceMonitor()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('device_monitor', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

        if not credentials:
            exit(1)

        gatekeeper = CognitoLoginManager()
        auth = gatekeeper.user_login(credentials)

        if not auth.is_ok():
            logger.error("login: %s." % auth.authentication_status.description)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # validation...

        if cmd.email is not None and cmd.exact_match and not Datum.is_email_address(cmd.email):
            logger.error("The email address '%s' is not valid." % cmd.email)
            exit(2)

        if cmd.device_tag is not None and cmd.exact_match and not CognitoDeviceCredentials.is_valid_tag(cmd.device_tag):
            logger.error("The device tag '%s' is not valid." % cmd.device_tag)
            exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        manager = DeviceMonitorSpecificationManager()
        byline_finder = BylineFinder()


        # ------------------------------------------------------------------------------------------------------------
        # identity...

        if cmd.device_tag:
            group = byline_finder.find_bylines_for_device(auth.id_token, cmd.device_tag)

            if not group:
                logger.error("device '%s' not found." % cmd.device_tag)
                exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.find:
            report = manager.find(auth.id_token, email_address=cmd.email, device_tag=cmd.device_tag,
                                  exact=cmd.exact_match)

        elif cmd.add:
            receipient = EmailRecipient(cmd.email, cmd.json_message)
            report = manager.add(auth.id_token, cmd.device_tag, receipient)

        elif cmd.suspend:
            report = manager.set_suspended(auth.id_token, cmd.device_tag, bool(cmd.is_suspended))

        elif cmd.delete:
            report = manager.remove(auth.id_token, cmd.email, device_tag=cmd.device_tag)


        # ------------------------------------------------------------------------------------------------------------
        # end...

        # report...
        if report is not None:
            print(JSONify.dumps(report, indent=cmd.indent))

        if cmd.find:
            logger.info('retrieved: %s' % len(report))

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPNotFoundException:
        logger.error("device '%s' not found." % cmd.device_tag)
        exit(1)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



"""
Created on 10 Mar 2020

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version
from scs_core.data.timedelta import Timedelta


# --------------------------------------------------------------------------------------------------------------------

class CmdCSVSegmentor(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog -m { [DD-]HH:MM[:SS] | :SS } [-i ISO] [-f FILE_PREFIX] "
                                                    "[-v]", version=version())

        # input...
        self.__parser.add_option("--max-interval", "-m", type="string", action="store", dest="max_interval",
                                 help="maximum permitted interval")

        self.__parser.add_option("--iso-path", "-i", type="string", action="store", default="rec", dest="iso",
                                 help="path for ISO 8601 datetime field (default 'rec')")

        # output...
        self.__parser.add_option("--file-prefix", "-f", type="string", action="store", dest="file_prefix",
                                 help="file prefix for contiguous CSVs")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.__opts.max_interval is None:
            return False

        if self.__args:
            return False

        return True


    def is_valid_interval(self):
        if self.__opts.max_interval is None:
            return True

        return self.max_interval is not None


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def max_interval(self):
        return Timedelta.construct_from_flag(self.__opts.max_interval)


    @property
    def iso(self):
        return self.__opts.iso


    @property
    def file_prefix(self):
        return self.__opts.file_prefix


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdCSVSegmentor:{max_interval:%s, iso:%s, file_prefix:%s, verbose:%s}" % \
               (self.__opts.max_interval, self.iso, self.file_prefix, self.verbose)



"""
Created on 24 Jan 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version
from scs_core.aws.security.cognito_user import CognitoUserIdentity


# --------------------------------------------------------------------------------------------------------------------

class CmdCognitoUsers(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        confirmations = ' | '.join(CognitoUserIdentity.status_codes())

        self.__parser = optparse.OptionParser(usage="%prog  [-c CREDENTIALS] "
                                                    "{ -F [{ -e EMAIL_ADDR | -l ORG_LABEL | -o CONFIRMATION | "
                                                    "-s { 0 | 1 } } }] [-m] "
                                                    "| -C -g GIVEN_NAME -f FAMILY_NAME -e EMAIL_ADDR "
                                                    "| -U EMAIL_ADDR [-g GIVEN_NAME] [-f FAMILY_NAME] [-e EMAIL_ADDR] "
                                                    "[-s { 0 | 1 }] "
                                                    "| -D EMAIL_ADDR } "
                                                    "[-i INDENT] [-v]",
                                              version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # operations...
        self.__parser.add_option("--Find", "-F", action="store_true", dest="find", default=False,
                                 help="list the users visible to me")

        self.__parser.add_option("--Create", "-C", action="store_true", dest="create", default=False,
                                 help="create a user")

        self.__parser.add_option("--Update", "-U", type="string", action="store", dest="update",
                                 help="update the user")

        self.__parser.add_option("--Delete", "-D", type="string", action="store", dest="delete",
                                 help="delete the user (superuser only)")

        # filters...
        self.__parser.add_option("--given-name", "-g", type="string", action="store", dest="given_name",
                                 help="given name")

        self.__parser.add_option("--family-name", "-f", type="string", action="store", dest="family_name",
                                 help="family name")

        self.__parser.add_option("--email", "-e", type="string", action="store", dest="email",
                                 help="email address")

        self.__parser.add_option("--org-label", "-l", type="string", action="store", dest="org_label",
                                 help="the organisation label")

        self.__parser.add_option("--confirmation-status", "-o", type="string", action="store",
                                 dest="confirmation_status",
                                 help="filter list by confirmation status { %s }" % confirmations)

        self.__parser.add_option("--enabled-status", "-s", type="int", action="store", dest="enabled",
                                 help="filter list by enabled status { 0 | 1 }")

        # output...
        self.__parser.add_option("--memberships", "-m", action="store_true", dest="memberships", default=False,
                                 help="show user's organisation memberships")

        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.find:
            count += 1

        if self.create:
            count += 1

        if self.update is not None:
            count += 1

        if self.delete is not None:
            count += 1

        if count != 1:
            return False

        count = 0

        if self.find:
            if self.org_label is not None:
                count += 1

            if self.email is not None:
                count += 1

            if self.confirmation_status is not None:
                count += 1

            if self.enabled is not None:
                count += 1

            if count > 1:
                return False

        if self.memberships and not self.find:
            return False

        if self.confirmation_status is not None and self.confirmation_status not in CognitoUserIdentity.status_codes():
            return False

        if self.enabled is not None and self.enabled not in (0, 1):
            return None

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    @property
    def find(self):
        return self.__opts.find


    @property
    def create(self):
        return self.__opts.create


    @property
    def update(self):
        return self.__opts.update


    @property
    def delete(self):
        return self.__opts.delete


    @property
    def given_name(self):
        return self.__opts.given_name


    @property
    def family_name(self):
        return self.__opts.family_name


    @property
    def email(self):
        return self.__opts.email


    @property
    def org_label(self):
        return self.__opts.org_label


    @property
    def confirmation_status(self):
        return self.__opts.confirmation_status


    @property
    def enabled(self):
        return self.__opts.enabled


    @property
    def memberships(self):
        return self.__opts.memberships


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdCognitoUsers:{credentials_name:%s, find:%s, create:%s, update:%s, delete:%s, " \
               "given_name:%s, family_name:%s, email:%s, org_label:%s, confirmation_status:%s, " \
               "enabled:%s, memberships:%s, indent:%s, verbose:%s}" % \
               (self.credentials_name, self.find, self.create, self.update, self.delete,
                self.given_name, self.family_name, self.email, self.org_label, self.confirmation_status,
                self.enabled, self.memberships, self.indent, self.verbose)



#!/usr/bin/env python3

"""
Created on 8 Aug 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

DESCRIPTION
Usage statistics are continuously collected for each web API by endpoint, date and user. The client_traffic utility is
used to query these statistics. Compulsory query fields are:

* -p PERIOD - should be in the form YYYY, YYYY-MM, or YYYY-MM-DD
* -u or -o CLIENT_1 [.. CLIENT_N] - specifies one or more email addresses, or one or more organisation labels

Optional query fields are:

* -e ENDPOINT - if specified, statistics for the given endpoint are reported, otherwise, all endpoints are reported

Other parameters are:

* --separate - (only permitted when specifying a single organisation) the organisation is broken down by individual
user members
* --aggregate - (only relevant when specifying a year or month period) statistics are aggregated to totals for the
period

Statistics are available only for the clients(s) that are visible to the user. Only organisation admins may view
statistics for whole organisations. Only superusers can view the statistics for all users.

SYNOPSIS
client_traffic.py [-c CREDENTIALS] [-e ENDPOINT] { -u | -o [-s] } -p PERIOD [-a] [-i INDENT] [-v]  [CLIENT_1..CLIENT_N]

EXAMPLES
client_traffic.py -c super -o Ricardo -e test1 -p 2023-08-22

client_traffic.py -v -c super -p 2024 -e TopicHistory -u production@southcoastscience.com | node.py -s | \
csv_writer.py -v traffic.csv

DOCUMENT EXAMPLE
[{"endpoint": "test1", "client": "MyOrg", "period": "2023-08-22", "queries": 99, "invocations": 99, "documents": 495}]

SEE ALSO
scs_analysis/cognito_user_credentials
"""

import sys

from scs_analysis.cmd.cmd_client_traffic import CmdClientTraffic

from scs_core.aws.client_traffic.client_traffic import ClientTrafficLocus
from scs_core.aws.client_traffic.client_traffic_finder import ClientTrafficFinder
from scs_core.aws.client_traffic.client_traffic_intercourse import ClientTrafficRequest

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager
from scs_core.aws.security.cognito_user_finder import CognitoUserFinder
from scs_core.aws.security.organisation_manager import OrganisationManager

from scs_core.client.http_exception import HTTPException

from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    clients = []
    response = None
    report = None

    # ------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdClientTraffic()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('client_traffic', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

        if not credentials:
            exit(1)

        gatekeeper = CognitoLoginManager()
        auth = gatekeeper.user_login(credentials)

        if not auth.is_ok():
            logger.error("login: %s." % auth.authentication_status.description)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # validation...

        if not ClientTrafficLocus.is_valid_period(cmd.period):
            logger.error("the period '%s' is not valid." % cmd.period)
            exit(2)

        if not cmd.organisation and not cmd.user and not cmd.endpoint:
            logger.error("an endpoint must be specified if no user or organisation is specified.")
            exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        user_finder = CognitoUserFinder()
        organisation_finder = OrganisationManager()
        traffic_finder = ClientTrafficFinder()


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.user:
            for client in cmd.clients:
                user = user_finder.get_by_email(auth.id_token, client)

                # if user is None:
                #     logger.error("the user '%s' could not be found." % client)
                #     exit(2)

                email = client if user is None else user.email      # include API keys

                clients.append(email)

        elif cmd.organisation:
            for client in cmd.clients:
                organisation = organisation_finder.get_organisation_by_label(auth.id_token, client)

                if organisation is None:
                    logger.error("the organisation '%s' could not be found." % client)
                    exit(2)

                clients.append(organisation.label)

        else:
            clients = []


        # ------------------------------------------------------------------------------------------------------------
        # run...

        request = ClientTrafficRequest(cmd.endpoint, clients, cmd.period, cmd.aggregate)

        if cmd.organisation:
            if cmd.separate:
                report = traffic_finder.find_for_organisations_users(auth.id_token, request)
            else:
                report = traffic_finder.find_for_organisations(auth.id_token, request)

        elif cmd.user:
            report = traffic_finder.find_for_users(auth.id_token, request)

        else:
            report = traffic_finder.find_for_endpoint(auth.id_token, request)

        report = sorted(report)


        # ------------------------------------------------------------------------------------------------------------
        # end...

        # report...
        if report is not None:
            print(JSONify.dumps(report, indent=cmd.indent))
            logger.info('reports: %s' % len(report))

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



"""
Created on 15 Feb 2019

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdSampleISO8601(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog { -z | { -o | -f DATE_FORMAT } "
                                                    "[-t TIMEZONE_NAME [-u]] [-i ISO_PATH] "
                                                    "{ DATETIME_PATH [-n] | DATE_PATH TIME_PATH } } [-s] [-v]",
                                              version=version())

        # helper...
        self.__parser.add_option("--zones", "-z", action="store_true", dest="zones", default=False,
                                 help="list the available timezone names to stderr")

        # formats...
        self.__parser.add_option("--oad", "-o", action="store_true", dest="oad", default=False,
                                 help="datetime format is OLE Automation date")

        self.__parser.add_option("--format", "-f", type="string", action="store", dest="format",
                                 help="specifies format of input date string, e.g. YYYY-MM-DD")

        self.__parser.add_option("--no-time", "-n", action="store_true", dest="no_time", default=False,
                                 help="no time is input (use 24:00:00)")

        self.__parser.add_option("--timezone", "-t", type="string", action="store",
                                 dest="timezone", help="source timezone (default 'UTC')")

        self.__parser.add_option("--utc", "-u", action="store_true", dest="utc", default=False,
                                 help="shift timezone to UTC")

        self.__parser.add_option("--iso-path", "-i", type="string", action="store", default="rec", dest="iso",
                                 help="path for ISO 8601 datetime output (default 'rec')")

        # mode...
        self.__parser.add_option("--skip-malformed", "-s", action="store_true", dest="skip_malformed", default=False,
                                 help="ignore rows with malformed datetimes")

        # output...
        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.zones:
            return True

        if self.oad:
            return True

        if self.format is None:
            return False

        if self.utc and self.timezone is None:
            return False

        if not self.__args:
            return False

        if len(self.__args) < 1 or len(self.__args) > 2:
            return False

        if self.no_time and len(self.__args) != 1:
            return False

        return True


    def datetime_paths(self):
        if self.uses_datetime():
            return {self.iso, self.datetime_path}

        elif self.uses_date_time():
            return {self.iso, self.date_path, self.time_path}

        return {self.iso}


    def uses_datetime(self):
        return len(self.__args) == 1


    def uses_date_time(self):
        return len(self.__args) == 2


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def zones(self):
        return self.__opts.zones


    @property
    def oad(self):
        return self.__opts.oad


    @property
    def format(self):
        return self.__opts.format


    @property
    def no_time(self):
        return self.__opts.no_time


    @property
    def timezone(self):
        return self.__opts.timezone


    @property
    def utc(self):
        return self.__opts.utc


    @property
    def iso(self):
        return self.__opts.iso


    @property
    def skip_malformed(self):
        return self.__opts.skip_malformed


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def datetime_path(self):
        return self.__args[0] if self.uses_datetime() else None


    @property
    def date_path(self):
        return self.__args[0] if self.uses_date_time() else None


    @property
    def time_path(self):
        return self.__args[1] if self.uses_date_time() else None


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdSampleISO8601:{zones:%s, oad:%s, format:%s, no_time:%s, timezone:%s, utc:%s, iso:%s, " \
               "skip_malformed:%s, verbose:%s, datetime_paths:%s}" % \
               (self.zones, self.oad, self.format, self.no_time, self.timezone, self.utc, self.iso,
                self.skip_malformed, self.verbose, self.datetime_paths())



#!/usr/bin/env python3

"""
Created on 12 Sep 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

DESCRIPTION
The configuration_report utility is used to generate a configuration CSV file by remotely interrogating the specified
device. The utility should be run in the directory that is scanned for raw report CSVs, such as
/Users/bruno/gbb/Production/ConfigurationGenerator/RawConfigs

SYNOPSIS
configuration_report.py [-c CREDENTIALS] [-v] DEVICE_TAG_1 [..DEVICE_TAG_N]

EXAMPLES
configuration_report.py -v -c super scs-opc-245

SEE ALSO
scs_analysis/device_controller

scs_mfr/configuration
"""

import sys

from scs_analysis.cmd.cmd_configuration_report import CmdConfigurationReport

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_device import CognitoDeviceCredentials
from scs_core.aws.security.cognito_device_finder import CognitoDeviceFinder
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager

from scs_core.client.http_exception import HTTPNotFoundException

from scs_core.sys.command import Command
from scs_core.sys.filesystem import Filesystem
from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    logger = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdConfigurationReport()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('configuration_report', verbose=cmd.verbose)
        logger = Logging.getLogger()

        logger.info(cmd)


        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

        if not credentials:
            exit(1)

        gatekeeper = CognitoLoginManager()
        auth = gatekeeper.user_login(credentials)

        if not auth.is_ok():
            logger.error("login: %s." % auth.authentication_status.description)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        finder = CognitoDeviceFinder()
        clu = Command(verbose=cmd.verbose)


        # ------------------------------------------------------------------------------------------------------------
        # validation...

        for device_tag in cmd.device_tags:
            if not CognitoDeviceCredentials.is_valid_tag(device_tag):
                logger.error("the device tag '%s' is not valid." % device_tag)
                exit(2)

            try:
                finder.get_by_tag(auth.id_token, device_tag)
            except HTTPNotFoundException:
                logger.error("no device found for tag: '%s'." % device_tag)
                exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        credentials = ['-c', cmd.credentials_name] if cmd.credentials_name else []

        for device_tag in cmd.device_tags:
            output_file = 'conf-%s.csv' % device_tag
            cmd_args = ['device_controller.py'] + credentials + ['-t', device_tag, '-m', '"configuration -t"', '-s',
                                                                 '>', output_file]
            p = clu.s(cmd_args, abort_on_fail=False)

            if p.returncode != 0:
                Filesystem.rm(output_file)


        # ------------------------------------------------------------------------------------------------------------
        # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)



"""
Created on 27 Sep 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import time

from socket import gaierror

from AWSIoTPythonSDK.exception.operationTimeoutException import operationTimeoutException

from scs_analysis.handler.mqtt_reporter import MQTTReporter

from scs_core.aws.client.client_auth import ClientAuth
from scs_core.aws.client.mqtt_client import MQTTClient

from scs_core.comms.mqtt_conf import MQTTConf

from scs_core.data.queue_report import QueueReport, ClientStatus


# --------------------------------------------------------------------------------------------------------------------

class AWSMQTTPublisher(object):
    """
    classdocs
    """

    __CONNECT_TIME =                3.0         # seconds
    __CONNECT_RETRY_TIME =         10.0         # seconds


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, conf: MQTTConf, auth: ClientAuth, client: MQTTClient, reporter: MQTTReporter):
        """
        Constructor
        """
        self.__conf = conf
        self.__auth = auth
        self.__client = client
        self.__reporter = reporter

        # report...
        client_state = ClientStatus.INHIBITED if conf.inhibit_publishing else ClientStatus.WAITING
        self.__status = QueueReport(0, client_state, False)
        self.__report()


    # ----------------------------------------------------------------------------------------------------------------

    def connect(self):
        if self.__conf.inhibit_publishing:
            return

        # report...
        self.__status.client_state = ClientStatus.CONNECTING
        self.__report()

        # connect...
        while True:
            try:
                if self.__client.connect(self.__auth, debug=self.__conf.debug):
                    break

                self.__reporter.print("connect: failed")
                time.sleep(self.__CONNECT_RETRY_TIME)
                continue

            except (OSError, gaierror) as ex:
                self.__reporter.print("connect: %s" % ex)
                time.sleep(self.__CONNECT_RETRY_TIME)
                continue

        time.sleep(self.__CONNECT_TIME)

        # report...
        self.__reporter.print("connect: done")
        self.__status.client_state = ClientStatus.CONNECTED
        self.__report()


    def disconnect(self):
        if self.__conf.inhibit_publishing:
            return

        # disconnect...
        self.__client.disconnect()

        # report...
        self.__reporter.print("disconnect: done")
        self.__status.client_state = ClientStatus.WAITING
        self.__report()


    def publish(self, publication):
        if self.__conf.inhibit_publishing:
            return

        # report...
        self.__status.length = 1                # indicate that document(s) have been presented for publication

        # publish...
        while True:
            try:
                start = time.time()
                reached_paho = self.__client.publish(publication)
                elapsed = time.time() - start

                self.__reporter.print("paho: %s: %0.3f" % ("1" if reached_paho else "0", elapsed))
                break

            except operationTimeoutException:
                # report...
                self.__status.publish_success = False
                self.__report()

        # report...
        self.__status.publish_success = True
        self.__report()


    # ----------------------------------------------------------------------------------------------------------------

    def __report(self):
        self.__status.save(self.__conf.report_file)


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "AWSMQTTPublisher:{conf:%s, auth:%s, client:%s, reporter:%s, report:%s}" % \
               (self.__conf, self.__auth, self.__client, self.__reporter, self.__status)



#!/usr/bin/env python3

"""
Created on 29 Jun 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

DESCRIPTION
The alert utility is used to create, update, delete or find alert specifications.

Alerts take the form of emails. These are sent when a parameter falls below or above specified bounds, or when the
value is null (a null value is being reported, or no reports are available). The alert specification sets these bounds,
together with the aggregation period. One of two types of period may be specified:

* Recurring - a period of a given number of minutes, hours or days. Recurring periods are sampled back-to back.
* Diurnal - a daily period with given start time and end times. Diurnal periods are sampled immediately after the end
time.

In --find mode, results can be filtered by description, topic, field or email address. Finder matches are exact.

When doing an update, the cc list (-g flag) may be preceeded with:

* A - add the following email address(es)
* R - remove the following email address(es)

If neither indicator is used, then the specified email address(es) replace the current ones.

SYNOPSIS
alert.py { -z | [-c CREDENTIALS]  { -F | -R ID | -C | -U ID | -D ID } [-d DESCRIPTION] [-p TOPIC] [-f FIELD]
[-l LOWER] [-u UPPER] [-n { 0 | 1 }] [{ -r INTERVAL UNITS TIMEZONE | -t START END TIMEZONE }]
[-a { 0 | 1 }] [-s { 0 | 1 }] [-i INDENT] [-v] [-e EMAIL_ADDR] [-b { A | R } EMAIL_ADDR [-j]] }

EXAMPLES
alert.py -vi4 -c super -C -d be2-3-nightime-test -p south-coast-science-dev/development/loc/1/climate -f val.tmp \
-u 10 -n 1 -t 16:00 8:00 Europe/London -e bruno.beloff@southcoastscience.com -g jade.page@southcoastscience.com

DOCUMENT EXAMPLE (recurring)
{"id": 85, "description": "test", "topic": "south-coast-science-demo/brighton-urban/loc/1/particulates",
"field": "exg.val.pm2p5", "lower-threshold": null, "upper-threshold": 20.0, "alert-on-none": false,
"aggregation-period": {"type": "recurring", "interval": 1, "units": "M", "timezone": "Europe/London"},
"contiguous-alerts": false, "json-message": false, "creator-email-address": "bruno.beloff@southcoastscience.com",
"to": "bruno.beloff@southcoastscience.com", "cc-list": [], "suspended": false}

DOCUMENT EXAMPLE (diurnal)
{"id": 107, "description": "be2-3-nightime-test", "topic": "south-coast-science-dev/development/loc/1/climate",
"field": "val.tmp", "lower-threshold": null, "upper-threshold": 10.0, "alert-on-none": false,
"aggregation-period": {"type": "diurnal", "start": "20:00:00", "end": "09:50:00", "timezone": "Europe/London"},
"contiguous-alerts": true, "json-message": false, "creator-email-address": "production@southcoastscience.com",
"to": "bruno.beloff@southcoastscience.com", "cc-list": ["jade.page@southcoastscience.com"], "suspended": false}

SEE ALSO
scs_analysis/alert_status
scs_analysis/cognito_user_credentials

BUGS
The test-interval field is not currently in use, and is ignored.
"""

import json
import sys

from scs_analysis.cmd.cmd_alert import CmdAlert

from scs_core.aws.monitor.alert.alert import AlertSpecification

from scs_core.aws.monitor.alert.alert_specification_manager import AlertSpecificationManager
from scs_core.aws.manager.byline.byline_finder import BylineFinder

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager

from scs_core.client.http_exception import HTTPException, HTTPNotFoundException

from scs_core.data.datum import Datum
from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict
from scs_core.data.str import Str

from scs_core.email.email import EmailRecipient

from scs_core.location.timezone import Timezone

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    logger = None
    credentials = None
    auth = None
    report = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdAlert()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('alert', verbose=cmd.verbose)
        logger = Logging.getLogger()

        logger.info(cmd)


        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        if not cmd.list:
            credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

            if not credentials:
                exit(1)

            gatekeeper = CognitoLoginManager()
            auth = gatekeeper.user_login(credentials)

            if not auth.is_ok():
                logger.error("login: %s." % auth.authentication_status.description)
                exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        byline_finder = BylineFinder()
        specification_manager = AlertSpecificationManager()


        # ------------------------------------------------------------------------------------------------------------
        # validation...

        try:
            if cmd.id is not None:
                int(cmd.id)
        except (TypeError, ValueError):
            logger.error('the ID must be an integer.')
            exit(2)

        if cmd.email is not None and not Datum.is_email_address(cmd.email):
            logger.error("the email address '%s' is not valid." % cmd.email)
            exit(2)

        if cmd.bcc:
            if not Datum.is_email_address(cmd.bcc_email):
                logger.error("the email address '%s' is not valid." % cmd.bcc_email)
                exit(2)

        if not cmd.is_valid_start_time():
            logger.error("the start time is invalid.")
            exit(2)

        if not cmd.is_valid_end_time():
            logger.error("the end time is invalid.")
            exit(2)

        if not cmd.is_valid_timezone():
            logger.error("the timezone is invalid.")
            exit(2)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.list:
            for zone in Timezone.zones():
                print(zone, file=sys.stderr)
            exit(0)

        if cmd.find:
            response = specification_manager.find(auth.id_token, cmd.description, cmd.topic, cmd.field, None)
            filtered = [alert for alert in response.alerts if cmd.email in alert] if cmd.email else response.alerts

            report = sorted(filtered)

        if cmd.retrieve:
            report = specification_manager.retrieve(auth.id_token, cmd.id)

        if cmd.create:
            # validation...
            if not cmd.is_complete():
                logger.error("minimum parameters are topic, path, a threshold, and an aggregation period.")
                exit(2)

            byline = byline_finder.find_latest_byline_for_topic(auth.id_token, cmd.topic)

            if byline is None or cmd.topic != byline.topic:
                logger.error("the topic '%s' is not available." % cmd.topic)
                exit(2)

            message = PathDict(json.loads(byline.message))

            if not message.has_path(cmd.field):
                paths = Str.collection(message.paths())
                logger.error("the field '%s' is not available. Available fields are: %s." % (cmd.field, paths))
                exit(2)

            # create...
            contiguous_alerts = True if cmd.contiguous_alerts is None else bool(cmd.contiguous_alerts)
            to_email = credentials.email if cmd.email is None else cmd.email
            to = EmailRecipient(to_email, cmd.json_message)

            bcc_dict = {}

            alert = AlertSpecification(None, cmd.description, cmd.topic, cmd.field, cmd.lower_threshold,
                                       cmd.upper_threshold, cmd.alert_on_none, cmd.aggregation_period,
                                       None, contiguous_alerts, None, to, bcc_dict, bool(cmd.suspended))

            if not alert.has_valid_thresholds():
                logger.error("threshold values are invalid.")
                exit(2)

            if not alert.has_valid_aggregation_period():
                logger.error("the aggregation period is invalid.")
                exit(2)

            report = specification_manager.create(auth.id_token, alert)

        if cmd.update:
            #  validation...
            if cmd.topic is not None:
                logger.error("topic may not be changed.")
                exit(2)

            alert = specification_manager.retrieve(auth.id_token, cmd.id)

            if alert is None:
                logger.error("no alert found with ID %s." % cmd.id)
                exit(2)

            # update...
            field = alert.field if not cmd.field else cmd.field
            description = alert.description if not cmd.description else cmd.description
            lower_threshold = alert.lower_threshold if cmd.lower_threshold is None else cmd.lower_threshold
            upper_threshold = alert.upper_threshold if cmd.upper_threshold is None else cmd.upper_threshold
            alert_on_none = alert.alert_on_none if cmd.alert_on_none is None else bool(cmd.alert_on_none)
            aggregation_period = alert.aggregation_period if cmd.aggregation_period is None else cmd.aggregation_period
            contiguous_alerts = alert.contiguous_alerts if cmd.contiguous_alerts is None else cmd.contiguous_alerts
            suspended = alert.suspended if cmd.suspended is None else bool(cmd.suspended)
            to_email = alert.to.email_address if cmd.email is None else cmd.email
            to = EmailRecipient(to_email, cmd.json_message)

            bcc_dict = cmd.updated_bcc_dict(alert.bcc_dict)

            updated = AlertSpecification(alert.id, description, alert.topic, field, lower_threshold, upper_threshold,
                                         alert_on_none, aggregation_period, None, contiguous_alerts,
                                         alert.creator_email_address, to, bcc_dict, suspended)

            if not updated.has_valid_thresholds():
                logger.error("threshold values are invalid.")
                exit(2)

            if not updated.has_valid_aggregation_period():
                logger.error("the aggregation period is invalid.")
                exit(2)

            report = specification_manager.update(auth.id_token, updated)

        if cmd.delete:
            try:
                specification_manager.delete(auth.id_token, cmd.id)
            except HTTPNotFoundException:
                logger.error("no alert found with ID %s." % cmd.id)


        # ------------------------------------------------------------------------------------------------------------
        # end...

        # report...
        if report is not None:
            print(JSONify.dumps(report, indent=cmd.indent))

        if cmd.find:
            logger.info('retrieved: %s' % len(report))

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



"""
Created on 15 Apr 2020

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdSampleCollator(object):
    """unix command line handler"""

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog -x IND_PATH [-n NAME] -y DEP_PATH "
                                                    "[-l LOWER_BOUND] -u UPPER_BOUND -d DELTA [-v]",
                                              version=version())

        # compulsory...
        self.__parser.add_option("--ind-path", "-x", type="string", action="store", dest="ind_path",
                                 help="path to independent variable")

        self.__parser.add_option("--name", "-n", type="string", action="store", dest="name",
                                 help="name of the independent variable")

        self.__parser.add_option("--dep-path", "-y", type="string", action="store", dest="dep_path",
                                 help="path to dependent variable")

        self.__parser.add_option("--upper", "-u", type="int", action="store", dest="upper",
                                 help="upper bound of dataset")

        self.__parser.add_option("--delta", "-d", type="int", action="store", dest="delta",
                                 help="width of column domain")

        # optional...
        self.__parser.add_option("--lower", "-l", type="int", action="store", dest="lower", default=0,
                                 help="lower bound of dataset (default 0)")

        # output...
        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.ind_path is None or self.dep_path is None or \
                self.lower is None or self.upper is None or self.delta is None:
            return False

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def ind_path(self):
        return self.__opts.ind_path


    @property
    def name(self):
        return self.__opts.name


    @property
    def dep_path(self):
        return self.__opts.dep_path


    @property
    def lower(self):
        return self.__opts.lower


    @property
    def upper(self):
        return self.__opts.upper


    @property
    def delta(self):
        return self.__opts.delta


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdSampleCollator:{ind_path:%s, name:%s, dep_path:%s, lower:%s, upper:%s, delta:%s, verbose:%s}" % \
               (self.ind_path, self.name, self.dep_path, self.lower, self.upper, self.delta, self.verbose)



"""
Created on 22 Feb 2019

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdCSVJoin(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-t TYPE] [-i] [-v] -l PREFIX PK FILENAME "
                                                    "-r PREFIX PK FILENAME", version=version())

        # input...
        self.__parser.add_option("--left", "-l", type="string", nargs=3, action="store", dest="left",
                                 help="output path prefix, primary key and filename for left-hand set")

        self.__parser.add_option("--right", "-r", type="string", nargs=3, action="store", dest="right",
                                 help="output path prefix, primary key and filename for right-hand set")

        # mode...
        self.__parser.add_option("--type", "-t", type="string", action="store", dest="type", default='INNER',
                                 help="{ 'INNER' | 'LEFT' | 'RIGHT' | 'FULL' } (default 'INNER')")

        self.__parser.add_option("--iso8601", "-i", action="store_true", dest="iso8601", default=False,
                                 help="interpret the primary key as an ISO 8601 datetime")

        # output...
        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.__opts.left is None or self.__opts.right is None:
            return False

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def type(self):
        return self.__opts.type


    @property
    def left_prefix(self):
        return None if self.__opts.left is None else self.__opts.left[0]


    @property
    def left_pk(self):
        return None if self.__opts.left is None else self.__opts.left[1]


    @property
    def left_filename(self):
        return None if self.__opts.left is None else self.__opts.left[2]


    @property
    def right_prefix(self):
        return None if self.__opts.right is None else self.__opts.right[0]


    @property
    def right_pk(self):
        return None if self.__opts.right is None else self.__opts.right[1]


    @property
    def right_filename(self):
        return None if self.__opts.right is None else self.__opts.right[2]


    @property
    def iso8601(self):
        return self.__opts.iso8601


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdCSVJoin:{type:%s, left:%s, right:%s, iso8601:%s, verbose:%s}" % \
               (self.type, self.__opts.left, self.__opts.right, self.iso8601, self.verbose)



"""
Created on 15 Feb 2019

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version
from scs_core.gas.gas import Gas


# --------------------------------------------------------------------------------------------------------------------

class CmdSampleGasConcentration(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-v] GAS DENSITY_PATH T_PATH [{P_PATH | -p PRESSURE}]",
                                              version=version())

        # mode...
        self.__parser.add_option("--pressure", "-p", type="float", action="store", dest="pressure",
                                 default=Gas.STP_PRESSURE, help="assume constant atmospheric pressure in kPA "
                                                                "(default %s)" % Gas.STP_PRESSURE)

        # output...
        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if len(self.__args) < 3:
            return False

        if self.pressure is not None and self.p_path is not None:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def pressure(self):
        return self.__opts.pressure


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def gas(self):
        return self.__args[0] if len(self.__args) > 0 else None


    @property
    def density_path(self):
        return self.__args[1] if len(self.__args) > 1 else None


    @property
    def t_path(self):
        return self.__args[2] if len(self.__args) > 2 else None


    @property
    def p_path(self):
        return self.__args[3] if len(self.__args) > 3 else None


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdSampleGasConcentration:{pressure:%s, verbose:%s, gas:%s, density_path:%s, t_path:%s, p_path:%s}" % \
               (self.pressure, self.verbose, self.gas, self.density_path, self.t_path, self.p_path)



"""
Created on 17 Apr 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdDeviceController(object):
    """unix message line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-c CREDENTIALS] -t DEVICE_TAG "
                                                    "[{ [-w] [-i INDENT] | [-s] -m CMD_TOKENS }]] [-v] ",
                                              version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        self.__parser.add_option("--device-tag", "-t", type="string", action="store", dest="device_tag",
                                 help="the device tag")

        # mode...
        self.__parser.add_option("--message", "-m", type="string", action="store", dest="message",
                                 help="send the given command(s)")

        # output...
        self.__parser.add_option("--wrapper", "-w", action="store_true", dest="wrapper", default=False,
                                 help="report message wrapper")

        self.__parser.add_option("--std", "-s", action="store_true", dest="std", default=False,
                                 help="write to stderr and stdout")

        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.std and (self.indent is not None or self.wrapper):
            return False

        if (self.wrapper or self.indent or self.std) and not self.message:
            return False

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------
    # properties: identity

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    @property
    def device_tag(self):
        return self.__opts.device_tag


    # ----------------------------------------------------------------------------------------------------------------
    # properties: mode

    @property
    def message(self):
        return self.__opts.message


    # ----------------------------------------------------------------------------------------------------------------
    # properties: output

    @property
    def wrapper(self):
        return self.__opts.wrapper


    @property
    def std(self):
        return self.__opts.std


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def commands(self):
        return self.__args


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdDeviceController:{credentials_name:%s, device_tag:%s, message:%s, wrapper:%s, std:%s, " \
               "indent:%s, verbose:%s}" % \
               (self.credentials_name, self.device_tag, self.message, self.wrapper, self.std,
                self.indent, self.verbose)



#!/usr/bin/env python3

"""
Created on 23 Aug 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_regression utility computes a linear regression for a stream of data delivered on stdin.

Input data is typically in the form of a JSON document. A command parameter specifies the path to the node within
the document that is to be averaged. The node is typically a leaf node integer or float. The output of the
sample_regression utility includes the last source value, slope and intercept.

SYNOPSIS
sample_regression.py [-t TALLY] [-p PRECISION] [-v] [PATH]

EXAMPLES
aws_topic_history.py -m60 /orgs/south-coast-science-demo/brighton/loc/1/gases | \
sample_regression.py -t360 -p3 val.CO.cnc

DOCUMENT EXAMPLE - INPUT
{"tag": "scs-bgx-401", "rec": "2018-03-27T09:54:41.042+00:00", "val": {
"NO2": {"weV": 0.29563, "aeV": 0.280879, "weC": 0.009569, "cnc": 61.0},
"Ox": {"weV": 0.406819, "aeV": 0.387443, "weC": -0.010706, "cnc": 34.7},
"NO": {"weV": 0.319692, "aeV": 0.292129, "weC": 0.028952, "cnc": 165.5},
"CO": {"weV": 0.395819, "aeV": 0.289317, "weC": 0.113108, "cnc": 311.3},
"sht": {"hmd": 82.4, "tmp": 12.6}}}

DOCUMENT EXAMPLE - OUTPUT
{"rec": "2018-03-27T16:10:51.033+00:00", "val": {"CO": {"cnc": {"src": 202.5, "slope": 0.039, "intercept": 226.217}}}}
"""

import sys

from scs_analysis.cmd.cmd_sample_tally import CmdSampleTally

from scs_analysis.handler.sample_regression import SampleRegression

from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleTally()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    if cmd.verbose:
        print("sample_regression: %s" % cmd, file=sys.stderr)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # resources...

        sampler = SampleRegression(cmd.path, cmd.tally, cmd.precision)

        if cmd.verbose:
            print("sample_regression: %s" % sampler, file=sys.stderr)
            sys.stderr.flush()


        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            datum = PathDict.construct_from_jstr(line)

            if datum is None:
                continue

            document_count += 1

            regression = sampler.datum(datum)

            if regression is not None:
                print(JSONify.dumps(regression))
                sys.stdout.flush()

            processed_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    finally:
        if cmd.verbose:
            print("sample_regression: documents: %d processed: %d" % (document_count, processed_count),
                  file=sys.stderr)



"""
Created on 24 Oct 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version
from scs_core.data.timedelta import Timedelta


# --------------------------------------------------------------------------------------------------------------------

class CmdSampleAggregate(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-c HH:MM:SS [-x] [-r { [DD-]HH:MM[:SS]] | :SS }]] "
                                                    "[-m] [-i ISO] [-v] [PATH_1..PATH_N]", version=version())

        # operation...
        self.__parser.add_option("--checkpoint", "-c", type="string", action="store", dest="checkpoint",
                                 help="a time specification i.e. **:/05:00")

        # mode...
        self.__parser.add_option("--min-max", "-m", action="store_true", dest="min_max", default=False,
                                 help="report min and max in addition to midpoint")

        self.__parser.add_option("--iso-path", "-i", type="string", action="store", default="rec", dest="iso",
                                 help="path for ISO 8601 datetime field (default 'rec')")

        self.__parser.add_option("--rule", "-r", type="string", action="store", dest="rule",
                                 help="apply 75% rule with sampling INTERVAL")

        self.__parser.add_option("--exclude-remainder", "-x", action="store_true", dest="exclude_remainder",
                                 help="ignore data points after the last complete period")

        # output...
        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if self.checkpoint is None and (self.__opts.rule is not None or self.exclude_remainder):
            return False

        return True


    def is_valid_interval(self):
        if self.__opts.rule is None:
            return True

        return self.rule_interval is not None


    def ignore_rule(self):
        return self.__opts.rule is None


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def checkpoint(self):
        return self.__opts.checkpoint


    @property
    def min_max(self):
        return self.__opts.min_max


    @property
    def iso(self):
        return self.__opts.iso


    @property
    def rule_interval(self):
        return Timedelta.construct_from_flag(self.__opts.rule)


    @property
    def exclude_remainder(self):
        return self.__opts.exclude_remainder


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def nodes(self):
        return self.__args


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdSampleAggregate:{checkpoint:%s, min_max:%s, iso:%s, rule_interval:%s, exclude_remainder:%s, " \
               "verbose:%s, nodes:%s}" %  \
               (self.checkpoint, self.min_max, self.iso, self.rule_interval, self.exclude_remainder,
                self.verbose, self.nodes)



"""
Created on 18 Jan 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdOrganisationUsers(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog  [-c CREDENTIALS] { -F [{ -e EMAIL | -l ORG_LABEL }] | "
                                                    "-R -e EMAIL -l ORG_LABEL | "
                                                    "-C -e EMAIL -l ORG_LABEL -o { 0 | 1 } -d { 0 | 1 } | "
                                                    "-U -e EMAIL -l ORG_LABEL [-o { 0 | 1 }] [-d { 0 | 1 }] "
                                                    "[-s { 0 | 1 }] | "
                                                    "-D -e EMAIL -l ORG_LABEL } "
                                                    "[-i INDENT] [-v]", version=version())

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # operations...
        self.__parser.add_option("--Find", "-F", action="store_true", dest="find", default=False,
                                 help="find users for the given username or organisation")

        self.__parser.add_option("--Retrieve", "-R", action="store_true", dest="retrieve", default=False,
                                 help="retrieve the user for the given username and organisation")

        self.__parser.add_option("--Create", "-C", action="store_true", dest="create", default=False,
                                 help="create a user")

        self.__parser.add_option("--Update", "-U", action="store_true", dest="update", default=False,
                                 help="Update the user")

        self.__parser.add_option("--Delete", "-D", action="store_true", dest="delete", default=False,
                                 help="delete the user for the given username and organisation")

        # fields...
        self.__parser.add_option("--email", "-e", type="string", action="store", dest="email",
                                 help="the user's email address (exact match)")

        self.__parser.add_option("--org-label", "-l", type="string", action="store", dest="org_label",
                                 help="the organisation label")

        self.__parser.add_option("--org-admin", "-o", type="int", action="store", dest="org_admin",
                                 help="the organisation administrator status")

        self.__parser.add_option("--device-admin", "-d", type="int", action="store", dest="device_admin",
                                 help="the device administrator status")

        self.__parser.add_option("--suspended", "-s", type="int", action="store", dest="suspended",
                                 help="the suspended status")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.find:
            count += 1

        if self.retrieve:
            count += 1

        if self.create:
            count += 1

        if self.update:
            count += 1

        if self.delete:
            count += 1

        if count != 1:
            return False

        if self.org_admin is not None and self.org_admin != 0 and self.org_admin != 1:
            return False

        if self.device_admin is not None and self.device_admin != 0 and self.device_admin != 1:
            return False

        if self.suspended is not None and self.suspended != 0 and self.suspended != 1:
            return False

        if self.find and bool(self.email) and bool(self.org_label):
            return False

        if (self.retrieve or self.create or self.update or self.delete) and \
                (self.email is None or self.org_label is None):
            return False

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    @property
    def find(self):
        return self.__opts.find


    @property
    def retrieve(self):
        return self.__opts.retrieve


    @property
    def create(self):
        return self.__opts.create


    @property
    def update(self):
        return self.__opts.update


    @property
    def delete(self):
        return self.__opts.delete


    @property
    def email(self):
        return self.__opts.email


    @property
    def org_label(self):
        return self.__opts.org_label


    @property
    def org_admin(self):
        return self.__opts.org_admin


    @property
    def device_admin(self):
        return self.__opts.device_admin


    @property
    def suspended(self):
        return self.__opts.suspended


    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdOrganisationUsers:{credentials_name:%s, find:%s, create:%s, update:%s, delete:%s, " \
               "email:%s, org_label:%s, org_admin:%s, device_admin:%s, suspended:%s, " \
               "indent:%s, verbose:%s}" % \
               (self.credentials_name, self.find, self.create, self.update, self.delete,
                self.email, self.org_label, self.__opts.org_admin, self.__opts.device_admin, self.__opts.suspended,
                self.indent, self.verbose)



#!/usr/bin/env python3

"""
Created on 31 Jul 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

DESCRIPTION
The baseline_conf utility is used to specify the operating parameters of the baseline utility.

Aggregation periods must be whole divisors of one hour, i.e. 1, 2, 3, 4, 5, 6, 10, 12, 15, 20, or 30 minutes.

Note that if the start time is a later hour than the end time, then the start time is interpreted as that hour
on the previous day.

Any number of separate baseline_conf files may be stored. The --list flag lists those that are available.

SYNOPSIS
baseline_conf.py [-a] { -z | -l | -n FROM TO | -c NAME [-s START] [-e END] [-p AGGREGATION] [-t TIMEZONE]
[-g GAS MINIMUM] [-r GAS] } [-i INDENT] [-v]

EXAMPLES
./baseline_conf.py -c freshfield -s 23:00 -e 08:00 -a 5 -t Europe/London -g NO2 10

DOCUMENT EXAMPLE
{"sample-period": {"type": "diurnal", "start": "23:00:00", "end": "08:00:00", "timezone": "Europe/London"},
"aggregation-period": {"type": "recurring", "interval": 5, "units": "M", "timezone": "Europe/London"},
"minimums": {"CO": 300, "CO2": 420, "H2S": 5, "NO": 10, "NO2": 10, "Ox": 50, "SO2": 5, "VOC": 250}}

FILES
~/SCS/conf/baseline_conf/NAME_baseline_conf.json

SEE ALSO
scs_analysis/baseline
scs_dev/gases_sampler
"""

import sys

from scs_analysis.cmd.cmd_baseline_conf import CmdBaselineConf

from scs_core.aws.client.access_key import AccessKey
from scs_core.aws.client.client import Client
from scs_core.aws.manager.s3_manager import S3PersistenceManager

from scs_core.data.diurnal_period import DiurnalPeriod
from scs_core.data.json import JSONify
from scs_core.data.recurring_period import RecurringMinutes, RecurringPeriod

from scs_core.estate.baseline_conf import BaselineConf

from scs_core.location.timezone import Timezone

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    conf = None
    key = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdBaselineConf()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('baseline_conf', verbose=cmd.verbose)
        logger = Logging.getLogger()

        logger.info(cmd)


        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        # PersistenceManager...
        if cmd.aws:
            if not AccessKey.exists(Host):
                logger.error('AccessKey not available.')
                exit(1)

            try:
                key = AccessKey.load(Host, encryption_key=AccessKey.password_from_user())
            except (KeyError, ValueError):
                logger.error('incorrect password.')
                exit(1)

            client = Client.construct('s3', key)
            resource_client = Client.resource('s3', key)

            persistence_manager = S3PersistenceManager(client, resource_client)

        else:
            persistence_manager = Host


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        # GasModelConf...
        if not cmd.duplicate():
            conf = BaselineConf.load(persistence_manager, name=cmd.conf_name, skeleton=cmd.set())


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.zones:
            for zone in Timezone.zones():
                print(zone, file=sys.stderr)
            exit(0)

        if cmd.list:
            for conf_name in BaselineConf.list(persistence_manager):
                print(conf_name, file=sys.stderr)
            exit(0)

        if cmd.duplicate():
            source = BaselineConf.load(persistence_manager, name=cmd.duplicate_from)

            if source is None:
                logger.error("no configuration found for '%s'" % cmd.duplicate_from)
                exit(2)

            conf = source.duplicate(cmd.duplicate_to)
            conf.save(persistence_manager)

        if cmd.set():
            if conf is None and not cmd.is_complete():
                logger.error("no configuration is stored - you must therefore set all fields.")
                exit(2)

            if cmd.timezone and not Timezone.is_valid(cmd.timezone):
                logger.error("unrecognised timezone name: %s." % cmd.timezone)
                exit(2)

            if cmd.interval and cmd.interval not in RecurringMinutes.valid_intervals():
                logger.error("aggregation period must be one of: %s." % str(RecurringMinutes.valid_intervals()))
                exit(2)

            if cmd.start is not None and not DiurnalPeriod.is_valid_time(cmd.start):
                logger.error("the start time is invalid.")
                exit(2)

            if cmd.end is not None and not DiurnalPeriod.is_valid_time(cmd.end):
                logger.error("the end time is invalid.")
                exit(2)

            start_time_str = str(conf.sample_period.start_time) if cmd.start is None else cmd.start
            end_time_str = str(conf.sample_period.end_time) if cmd.end is None else cmd.end
            timezone_str = str(conf.timezone) if cmd.timezone is None else cmd.timezone

            sample_period = DiurnalPeriod.construct(start_time_str, end_time_str, timezone_str)

            interval = conf.aggregation_period.interval if cmd.interval is None else cmd.interval

            aggregation_period = RecurringPeriod.construct(interval, 'M', timezone_str)

            if sample_period.start_time == sample_period.end_time:
                logger.error("the start and end hours may not be the same.")
                exit(2)

            minimums = {} if conf is None else conf.minimums

            conf = BaselineConf(cmd.conf_name, sample_period, aggregation_period, minimums)

            if cmd.set_gas_name:
                if cmd.set_gas_name not in BaselineConf.supported_gases():
                    logger.error("the gas '%s' is not supported." % cmd.set_gas_name)
                    exit(2)

                conf.set_minimum(cmd.set_gas_name, cmd.set_gas_minimum)

            if cmd.remove_gas:
                conf.remove_minimum(cmd.remove_gas)

            conf.save(persistence_manager)


        # ------------------------------------------------------------------------------------------------------------
        # end...

        if conf is not None:
            print(JSONify.dumps(conf, indent=cmd.indent))

    except KeyboardInterrupt:
        print(file=sys.stderr)



"""
Created on 28 Jun 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdDeviceMonitor(object):
    """unix command line handler"""

    # --------------------------------------------------------------------------------------------------------------------

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-c CREDENTIALS] "
                                                    "{ -F [{ -e EMAIL_ADDR | -t DEVICE_TAG } [-x]] | "
                                                    "-A EMAIL_ADDR DEVICE_TAG [-j] | -S DEVICE_TAG { 0 | 1 } | "
                                                    "-D EMAIL_ADDR -t DEVICE_TAG } "
                                                    "[-i INDENT] [-v]", version=version())

        # [{ -t DEVICE_TAG | -f }]

        # identity...
        self.__parser.add_option("--credentials", "-c", type="string", action="store", dest="credentials_name",
                                 help="the stored credentials to be presented")

        # operations...
        self.__parser.add_option("--find", "-F", action="store_true", dest="find", default=False,
                                 help="find monitoring (for email address or device)")

        self.__parser.add_option("--add", "-A", type="string", nargs=2, action="store", dest="add",
                                 help="add email address to device")

        self.__parser.add_option("--suspend", "-S", type="string", nargs=2, action="store", dest="suspend",
                                 help="suspend monitoring of the device")

        self.__parser.add_option("--delete", "-D", type="string", action="store", dest="delete",
                                 help="delete email address (from device or throughout)")

        # self.__parser.add_option("--force", "-f", type="string", action="store", dest="force",
        #                          help="force deletion from multiple devices")

        # fields...
        self.__parser.add_option("--json-message", "-j", action="store_true", dest="json_message", default=False,
                                 help="message body is JSON")

        # filters...
        self.__parser.add_option("--email", "-e", type="string", action="store", dest="email",
                                 help="email address")

        self.__parser.add_option("--device-tag", "-t", type="string", action="store", dest="device_tag",
                                 help="device tag")

        self.__parser.add_option("--exactly", "-x", action="store_true", dest="exact_match", default=False,
                                 help="exact match for tag")

        # output...
        self.__parser.add_option("--indent", "-i", type="int", action="store", dest="indent",
                                 help="pretty-print the output with INDENT")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        count = 0

        if self.find:
            count += 1

        if self.add:
            count += 1

        if self.suspend:
            count += 1

        if self.delete:
            count += 1

        if count != 1:
            return False

        if not self.add and self.json_message:
            return False

        if self.suspend and self.__opts.suspend[1] not in [None, '0', '1']:
            return False

        if self.__args:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------
    # properties: identity...

    @property
    def credentials_name(self):
        return self.__opts.credentials_name


    # ----------------------------------------------------------------------------------------------------------------
    # properties: operations...

    @property
    def find(self):
        return self.__opts.find


    @property
    def add(self):
        return bool(self.__opts.add)


    @property
    def suspend(self):
        return bool(self.__opts.suspend)


    @property
    def is_suspended(self):
        return bool(int(self.__opts.suspend[1])) if self.suspend else None


    @property
    def delete(self):
        return bool(self.__opts.delete)


    # ----------------------------------------------------------------------------------------------------------------
    # properties: fields...

    @property
    def json_message(self):
        return self.__opts.json_message


    # ----------------------------------------------------------------------------------------------------------------
    # properties: filters...

    @property
    def email(self):
        if self.add:
            return self.__opts.add[0]

        if self.delete:
            return self.__opts.delete

        return self.__opts.email


    @property
    def device_tag(self):
        if self.add:
            return self.__opts.add[1]

        if self.suspend:
            return self.__opts.suspend[0]

        return self.__opts.device_tag


    @property
    def exact_match(self):
        return self.__opts.exact_match


    # ----------------------------------------------------------------------------------------------------------------
    # properties: output...

    @property
    def indent(self):
        return self.__opts.indent


    @property
    def verbose(self):
        return self.__opts.verbose


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdDeviceMonitor:{credentials_name:%s, find:%s, add:%s, suspend:%s, delete:%s, " \
               "email:%s, device_tag:%s, json_message:%s, exact_match:%s, " \
               "indent:%s, verbose:%s}" % \
               (self.credentials_name, self.__opts.find, self.__opts.add, self.__opts.suspend, self.__opts.delete,
                self.__opts.email, self.__opts.device_tag, self.json_message, self.exact_match,
                self.indent, self.verbose)



#!/usr/bin/env python3

"""
Created on 19 Aug 2016

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The csv_writer utility is used to convert from JSON format to comma-separated value (CSV) format.

The path into the JSON document is used to name the column in the header row: dictionary fields are separated
from their container by a period ('.') character, and array members are separated by a colon (':') character.

The --quote-all flag forces quote ('"') characters around all cell values. This is useful when long numeric
codes such as IMEI numbers are included in the data, such codes should not be interpreted as floats by
spreadsheet applications. The default behaviour is to quote only where necessary.

default mode:

All the leaf nodes of the first JSON document are included in the CSV. If subsequent JSON documents in the input stream
contain fields that were not in this first document, these extra fields are ignored. If subsequent JSON documents
do not contain a field that is in the header, then this field is given the null value.

header-scan mode:

All input documents are scanned in order to build an inclusive hearer row. Any documents that do not contain a header
fields are given a null value for that field. Any values bound to paths that become internal nodes are discarded.
Warning: the header-scan mode requires memory proportional to the size of its input.

SYNOPSIS
csv_writer.py [{ -a | -x | -s }] [-q] [-e] [-v] [FILENAME]

EXAMPLES
socket_receiver.py | csv_writer.py temp.csv -e

DOCUMENT EXAMPLE - INPUT
{"tag": "scs-ap1-6", "rec": "2018-04-04T14:50:27.641+00:00", "val": {"hmd": 59.6, "tmp": 23.8}}

DOCUMENT EXAMPLE - OUTPUT
tag,rec,val.hmd,val.tmp
scs-ap1-6,2018-04-04T14:50:38.394+00:00,59.7,23.8

SEE ALSO
scs_analysis/csv_logger
scs_analysis/csv_reader
"""

import sys

from scs_analysis.cmd.cmd_csv_writer import CmdCSVWriter

from scs_core.csv.csv_writer import CSVWriter


# TODO: fix append mode with no newline at the end of the CSV?
# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    writer = None

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdCSVWriter()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    if cmd.verbose:
        print("csv_writer: %s" % cmd, file=sys.stderr)
        sys.stderr.flush()

    try:
        # ------------------------------------------------------------------------------------------------------------
        # resources...

        writer = CSVWriter(filename=cmd.filename, append=cmd.append, exclude_header=cmd.exclude_header,
                           header_scan=cmd.header_scan, quote_all=cmd.quote_all)

        if cmd.verbose:
            print("csv_writer: %s" % writer, file=sys.stderr)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            jstr = line.strip()

            document_count += 1

            if not writer.write(jstr):
                continue

            # echo...
            if cmd.echo:
                print(jstr)
                sys.stdout.flush()

            processed_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    finally:
        if writer is not None:
            writer.close()

        if cmd.verbose:
            print("csv_writer: documents: %d processed: %d" % (document_count, processed_count), file=sys.stderr)



#!/usr/bin/env python3

"""
Created on 3 Mar 2019

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_duplicates utility is used to find duplicate values in a sequence of input JSON documents, optionally
for a specified node path. It is particularly useful in searching for duplicate recording datetimes.

If an input document does not contain the specified path, then it is ignored.

In the default mode, the utility outputs the rows that were duplicates (or contained duplicate field values). If the
--exclude flag is set, then sample_duplicates generates a version of the input data that contains no duplicates.

In the --counts mode, the output report is sequence of JSON dictionaries with a field for each value where duplicates
were found, whose value is the number of matching documents.

SYNOPSIS
sample_duplicates.py [{ -x | -c }] [-v] [PATH]

EXAMPLES
csv_reader.py climate.csv | sample_duplicates.py -v val.hmd

DOCUMENT EXAMPLE - OUTPUT
default mode:
{"val": {"hmd": 17.5, "tmp": 25.7}, "rec": "2019-02-25T15:28:18Z", "tag": "scs-bgx-303"}
{"val": {"hmd": 17.5, "tmp": 25.7}, "rec": "2019-02-25T15:31:18Z", "tag": "scs-bgx-303"}

counts mode:
{"17.5": 2}
"""

import sys

from scs_analysis.cmd.cmd_sample_duplicates import CmdSampleDuplicates

from scs_core.data.duplicates import Duplicates
from scs_core.data.path_dict import PathDict

from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    dupes = None

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleDuplicates()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    Logging.config('sample_duplicates', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # resources...

        dupes = Duplicates()
        non_dupes = []

        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            jstr = line.strip()
            datum = PathDict.construct_from_jstr(jstr)

            if datum is None:
                continue

            document_count += 1

            if not datum.has_sub_path(cmd.path):
                continue

            is_duplicate = dupes.test(document_count, datum.node(cmd.path), datum)

            if not cmd.counts:
                if cmd.exclude:
                    if not is_duplicate:
                        non_dupes.append(jstr)
                else:
                    if is_duplicate:
                        print(jstr)

            processed_count += 1


        # ------------------------------------------------------------------------------------------------------------
        # report...

        if cmd.exclude:
            for datum in non_dupes:
                print(datum)

        if cmd.counts:
            for count in dupes.match_counts():
                print(count)


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    finally:
        logger.info("documents: %d processed: %d" % (document_count, processed_count))
        logger.info("values with duplicates: %d total duplicates: %d" % (dupes.matched_key_count, dupes.total_matches))



#!/usr/bin/env python3

"""
Created on 9 Feb 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The cognito_email utility is used to set or reset the user's password, or to request an email to permit
password operations. Which email message is sent depends on the state of the state of the user's account:

* UNCONFIRMED - the account creation email is re-sent
* CONFIRMED - a password reset code is sent and the user account enters the PASSWORD_RESET_REQUIRED state
* PASSWORD_RESET_REQUIRED - a password reset code is re-sent
* FORCE_CHANGE_PASSWORD - the temporary password is re-sent

Emails cannot be sent to users in the DISABLED state.

SYNOPSIS
cognito_email.py { -e | -s | -r } EMAIL [-v]

EXAMPLES
./cognito_email.py -r someone@me.com

SEE ALSO
scs_analysis/cognito_user_credentials
scs_analysis/cognito_user_identity
scs_analysis/cognito_users
"""

import sys

from scs_analysis.cmd.cmd_cognito_email import CmdCognitoEmail

from scs_core.aws.security.cognito_authentication import AuthenticationStatus
from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager
from scs_core.aws.security.cognito_password_manager import CognitoPasswordManager
from scs_core.aws.security.cognito_user import CognitoUserIdentity
from scs_core.aws.security.cognito_user_manager import CognitoUserCreator

from scs_core.client.http_exception import HTTPException, HTTPBadRequestException, HTTPGoneException, \
    HTTPNotFoundException, HTTPNotAllowedException, HTTPUnauthorizedException

from scs_core.data.datum import Datum

from scs_core.sys.logging import Logging

from scs_host.comms.stdio import StdIO


# --------------------------------------------------------------------------------------------------------------------

def do_password(do_set):
    new_password = StdIO.prompt("Enter new password")

    if not CognitoUserIdentity.is_valid_password(new_password):
        logger.error("The password must include lower and upper case, numeric and punctuation characters.")
        exit(1)

    try:
        if do_set:
            password_manager.do_set_password(cmd.email, new_password, auth.content.session)
        else:
            password_manager.do_reset_password(cmd.email, code, new_password)

    except HTTPNotFoundException:
        logger.error("no user could be found for email '%s'." % cmd.email)
        exit(1)

    except HTTPUnauthorizedException:
        logger.error("the user '%s' is disabled." % cmd.email)
        exit(1)

    except HTTPNotAllowedException:
        alternative = '--reset-password' if do_set else '--set-password'
        logger.error("the operation is not permitted in the current user state. Try %s?" % alternative)
        exit(1)

    except HTTPBadRequestException as error:
        if error.data == "Mismatch":
            logger.error("the code '%s' is not valid." % code)
            exit(1)

        if error.data == "Expired":
            logger.error("the code has expired.")
            exit(1)

        logger.error(repr(ex))


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    logger = None
    credentials = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdCognitoEmail()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('cognito_email', verbose=cmd.verbose)
        logger = Logging.getLogger()

        logger.info(cmd)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        password_manager = CognitoPasswordManager()
        user_manager = CognitoUserCreator()
        gatekeeper = CognitoLoginManager()


        # ------------------------------------------------------------------------------------------------------------
        # validation...

        if not Datum.is_email_address(cmd.email):
            logger.error("The email address is not valid.")
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.send_email:
            try:
                password_manager.send_email(cmd.email)
                logger.error("an email has been sent.")
                exit(0)

            except HTTPUnauthorizedException:
                logger.error("the user '%s' is disabled." % cmd.email)
                exit(1)

            except HTTPNotFoundException:
                logger.error("no user could be found for email '%s'." % cmd.email)
                exit(2)

        if cmd.confirm:
            confirmation_code = StdIO.prompt("Enter confirmation code")

            try:
                user_manager.confirm(cmd.email, confirmation_code)
                logger.error("OK")
                exit(0)

            except HTTPNotFoundException:
                logger.error("no user could be found for email '%s'." % cmd.email)
                exit(2)

            except HTTPGoneException:
                logger.error("the account has already been confirmed.")
                exit(1)

        if cmd.set_password:
            temporary_password = StdIO.prompt("Enter temporary password")

            # login
            credentials = CognitoClientCredentials(None, cmd.email, temporary_password, None)
            auth = gatekeeper.user_login(credentials)

            if auth.authentication_status != AuthenticationStatus.Challenge:
                logger.error(auth.authentication_status.description)
                exit(1)

            do_password(True)
            logger.error("the password has been set.")
            exit(0)

        if cmd.reset_password:
            code = StdIO.prompt("Enter verification code")

            do_password(False)
            logger.error("the password has been reset - update your stored credentials.")
            exit(0)


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



#!/usr/bin/env python3

"""
Created on 22 Nov 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The cognito_user_credentials utility is used to manage the AWS Cognito credentials on the user's computer. The
credentials are composed of an email address and a password. The password must be specified when the credentials are
created and is required when the credentials are accessed.

The JSON identity document managed by this utility is encrypted, and a password must be used to retrieve the document.
By default, the retrieval password is the same as the Cognito credentials password. However, a separate retrieval
password can be specified (in order, for example, to standardise the retrieval password across multiple Cognito
accounts).

The --credentials flag is only required where the user wishes to store multiple identities.

Setting the credentials is done interactively using the command line interface.

SYNOPSIS
cognito_user_credentials.py [{ -l | [-c CREDENTIALS] [{ -s | -p | -t | -d }] }] [-v]

EXAMPLES
./cognito_user_credentials.py -s

FILES
~/SCS/aws/cognito_user_credentials.json

DOCUMENT EXAMPLE
{"email": "production@southcoastscience.com", "password": "###", "retrieval-password": "###"}

SEE ALSO
scs_analysis/cognito_user_identity

RESOURCES
https://stackoverflow.com/questions/42568262/how-to-encrypt-text-with-a-password-in-python
"""

import sys

from scs_analysis.cmd.cmd_cognito_user_credentials import CmdCognitoUserCredentials

from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager
from scs_core.aws.security.cognito_user import CognitoUserIdentity

from scs_core.client.http_exception import HTTPException

from scs_core.data.datum import Datum
from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    logger = None
    credentials = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdCognitoUserCredentials()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('cognito_user_credentials', verbose=cmd.verbose)
        logger = Logging.getLogger()

        logger.info(cmd)


        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        if not (cmd.list or cmd.set or cmd.delete):
            credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        if cmd.list:
            for conf_name in sorted(CognitoClientCredentials.list(Host)):
                print(conf_name, file=sys.stderr)
            exit(0)

        elif cmd.set:
            credentials = CognitoClientCredentials.from_user(cmd.credentials_name)

            if not Datum.is_email_address(credentials.email):
                logger.error("The email address is not valid.")
                exit(1)

            if not CognitoUserIdentity.is_valid_password(credentials.password):
                logger.error("The password must include lower and upper case, numeric and punctuation characters.")
                exit(1)

            credentials.save(Host, encryption_key=credentials.retrieval_password)

        elif cmd.update_password:
            if credentials is None:
                logger.error("credentials not found.")
                exit(1)

            credentials = CognitoClientCredentials.from_user(cmd.credentials_name, existing_email=credentials.email)

            if not Datum.is_email_address(credentials.email):
                logger.error("The email address is not valid.")
                exit(1)

            if not CognitoUserIdentity.is_valid_password(credentials.password):
                logger.error("The password must include lower and upper case, numeric and punctuation characters.")
                exit(1)

            credentials.save(Host, encryption_key=credentials.retrieval_password)

        elif cmd.test:
            gatekeeper = CognitoLoginManager()

            if credentials is None:
                logger.error("no credentials are available")
                exit(1)

            logger.info(credentials)

            result = gatekeeper.user_login(credentials)
            logger.error(result.authentication_status.description)

            exit(0 if result.is_ok() else 1)

        elif cmd.delete:
            CognitoClientCredentials.delete(Host)


    # ----------------------------------------------------------------------------------------------------------------
    # end...

        if credentials:
            print(JSONify.dumps(credentials))

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



#!/usr/bin/env python3

"""
Created on 20 Apr 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

DESCRIPTION
The configuration_monitor utility is used to retrieve configuration information relating to one or more devices.
Flags enable the selection of either the latest recorded configuration for the device(s), or a history of
configuration changes. In the case of historical reports, either all the field values can be returned, or
only those that changed from the previous recording.

SYNOPSIS
configuration_monitor.py [-c CREDENTIALS] [-t TAG [-x]] { -l | -f | -d | -o } [-i INDENT] [-v]

EXAMPLES
configuration_monitor.py -t scs-bgx-401 -d | node.py -s | csv_writer.py -s

SEE ALSO
scs_analysis/configuration_csv
scs_analysis/configuration_monitor_check
scs_analysis/monitor_auth

scs_mfr/configuration
"""

import sys

from scs_analysis.cmd.cmd_configuration_monitor import CmdConfigurationMonitor
from scs_analysis.handler.batch_download_reporter import BatchDownloadReporter

from scs_core.aws.manager.configuration.configuration_finder import ConfigurationFinder
from scs_core.aws.security.cognito_client_credentials import CognitoClientCredentials
from scs_core.aws.security.cognito_login_manager import CognitoLoginManager

from scs_core.client.http_exception import HTTPException

from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging

from scs_host.sys.host import Host


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    logger = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # cmd...

        cmd = CmdConfigurationMonitor()

        if not cmd.is_valid():
            cmd.print_help(sys.stderr)
            exit(2)

        Logging.config('configuration_monitor', verbose=cmd.verbose)
        logger = Logging.getLogger()

        logger.info(cmd)


        # ------------------------------------------------------------------------------------------------------------
        # authentication...

        credentials = CognitoClientCredentials.load_for_user(Host, name=cmd.credentials_name)

        if not credentials:
            exit(1)

        gatekeeper = CognitoLoginManager()
        auth = gatekeeper.user_login(credentials)

        if not auth.is_ok():
            logger.error("login: %s." % auth.authentication_status.description)
            exit(1)


        # ------------------------------------------------------------------------------------------------------------
        # resources...

        # ConfigurationFinder...
        reporter = BatchDownloadReporter()
        finder = ConfigurationFinder(reporter=reporter)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        response = finder.find(auth.id_token, cmd.tag_filter, cmd.exact_match, cmd.response_mode())
        items = list(response)

        print(JSONify.dumps(sorted(items), indent=cmd.indent))
        logger.info('retrieved: %s' % len(items))


        # ------------------------------------------------------------------------------------------------------------
        # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except HTTPException as ex:
        logger.error(ex.error_report)
        exit(1)

    except Exception as ex:
        logger.error(ex.__class__.__name__)
        exit(1)



"""
Created on 2 Aug 2016

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdCSVReader(object):
    """unix command line handler"""

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog [-s] [-n] [-l LIMIT] [-a] [-v] [FILENAME_1 .. FILENAME_N]",
                                              version=version())

        # mode...
        self.__parser.add_option("--string", "-s", action="store_true", dest="string", default=False,
                                 help="interpret all values as strings")

        self.__parser.add_option("--nullify", "-n", action="store_true", dest="nullify", default=False,
                                 help="convert empty or \"NULL\" strings to nulls")

        self.__parser.add_option("--limit", "-l", type="int", action="store", dest="limit",
                                 help="output a maximum of LIMIT rows")

        # output...
        self.__parser.add_option("--array", "-a", action="store_true", dest="array", default=False,
                                 help="output JSON documents as array instead of a sequence")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def cast(self):
        return not self.string


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def string(self):
        return self.__opts.string


    @property
    def nullify(self):
        return self.__opts.nullify


    @property
    def limit(self):
        return self.__opts.limit


    @property
    def array(self):
        return self.__opts.array


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def filenames(self):
        return self.__args if len(self.__args) > 0 else [None]


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdCSVReader:{string:%s, nullify:%s, limit:%s, array:%s, verbose:%s, filenames:%s}" % \
               (self.string, self.nullify, self.limit, self.array, self.verbose, self.filenames)



#!/usr/bin/env python3

"""
Created on 24 Dec 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The aws_upload_interval utility is used to determine the end-to-end health of the data infrastructure. Specifically, it
reports on the time difference between a sense document being created on a device, and the same document being
ingested by the data storage system. Time difference is represented as:

DAYS-HOURS:MINUTES:SECONDS

The aws_upload_interval utility relies on the data format provided by the aws_topic_history utility - when invoked with
its -w ("wrapper") flag, the aws_topic_history output includes an "upload" field, indicating the date / time at which
the document was processed. The sense time is provided in the "payload.rec" field. The aws_upload_interval
utility reports on the difference between the two date / times in seconds.

The utility accepts input from stdin. If an input document is empty or is not well-formed JSON, the document is
ignored. If the document is well-formed JSON, but does not contain the required "upload" and "payload.rec" fields or
the fields are not in ISO 8601 format, then an error is reported, and the aws_upload_interval utility terminates.

SYNOPSIS
aws_upload_interval.py

EXAMPLES
./aws_topic_history.py unep/ethiopia/loc/1/climate -m1440 -w | ./aws_upload_interval.py

DOCUMENT EXAMPLE - INPUT
{"device": "scs-bbe-501", "topic": "unep/ethiopia/loc/1/climate", "upload": "2018-12-24T13:07:03Z",
"payload": {"val": {"hmd": 36.5, "tmp": 25.5}, "rec": "2018-12-24T13:07:01Z", "tag": "scs-bgx-501"}}

DOCUMENT EXAMPLE - OUTPUT
{"upload": "2018-12-24T13:07:03Z", "rec": "2018-12-24T13:07:01Z", "offset": "00-00:00:01"}

SEE ALSO
scs_analysis/aws_topic_history

RESOURCES
https://en.wikipedia.org/wiki/ISO_8601
"""

import sys

from scs_core.data.json import JSONify

from scs_core.aws.data.upload_interval import UploadInterval


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    interval = None

    try:
        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            jstr = line.strip()

            try:
                interval = UploadInterval.construct_from_jstr(jstr)

            except (KeyError, ValueError) as ex:
                print("aws_upload_interval: %s on: %s" % (repr(ex), jstr), file=sys.stderr)
                exit(1)

            if interval is None:
                continue

            # report...
            print(JSONify.dumps(interval))
            sys.stdout.flush()


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)



#!/usr/bin/env python3

"""
Created on 13 Oct 2016

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis

DESCRIPTION
The sample_average utility is used to compute an average value for a stream of data delivered on stdin. This can be an
average of all values, or a rolling average for a given number of values.

Input data is typically in the form of a JSON document. A command parameter specifies the path to the node within
the document that is to be averaged. The node is typically a leaf node integer or float. The output of the
sample_average utility includes the source value, and the average value.

SYNOPSIS
sample_average.py [-t TALLY] [-p PRECISION] [-v] [PATH]

EXAMPLES
aws_topic_history.py -m1 /orgs/south-coast-science-demo/brighton/loc/1/gases | sample_average.py -t3 -p1 val.CO.cnc

DOCUMENT EXAMPLE - INPUT
{"tag": "scs-bgx-401", "rec": "2018-03-27T09:54:41.042+00:00", "val": {
"NO2": {"weV": 0.29563, "aeV": 0.280879, "weC": 0.009569, "cnc": 61.0},
"Ox": {"weV": 0.406819, "aeV": 0.387443, "weC": -0.010706, "cnc": 34.7},
"NO": {"weV": 0.319692, "aeV": 0.292129, "weC": 0.028952, "cnc": 165.5},
"CO": {"weV": 0.395819, "aeV": 0.289317, "weC": 0.113108, "cnc": 311.3},
"sht": {"hmd": 82.4, "tmp": 12.6}}}

DOCUMENT EXAMPLE - OUTPUT
{"rec": "2018-03-27T10:01:21.028+00:00", "val": {"CO": {"cnc": {"src": 145.6, "avg": 168.2}}}}
"""

import sys

from scs_analysis.cmd.cmd_sample_tally import CmdSampleTally

from scs_core.data.average import Average
from scs_core.data.json import JSONify
from scs_core.data.path_dict import PathDict


# --------------------------------------------------------------------------------------------------------------------

class SampleAverage(object):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, path, tally, precision):
        """
        Constructor
        """
        self.__path = path
        self.__precision = precision

        self.__func = Average(tally)


    # ----------------------------------------------------------------------------------------------------------------

    def datum(self, sample):
        if not sample.has_path(self.__path):
            return None

        value = sample.node(self.__path)
        self.__func.append(value)

        avg = self.__func.mid()

        if avg is None:
            return None

        target = PathDict()

        if sample.has_path('rec'):
            target.copy(sample, 'rec')

        target.append(self.__path + '.src', value)
        target.append(self.__path + '.avg', round(avg, self.__precision))

        return target.node()


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "SampleAverage:{path:%s, precision:%s, func:%s}" % (self.__path, self.__precision, self.__func)


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    document_count = 0
    processed_count = 0

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdSampleTally()

    if not cmd.is_valid():
        cmd.print_help(sys.stderr)
        exit(2)

    if cmd.verbose:
        print("sample_average: %s" % cmd, file=sys.stderr)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # resources...

        sampler = SampleAverage(cmd.path, cmd.tally, cmd.precision)

        if cmd.verbose:
            print("sample_average: %s" % sampler, file=sys.stderr)
            sys.stderr.flush()


        # ------------------------------------------------------------------------------------------------------------
        # run...

        for line in sys.stdin:
            datum = PathDict.construct_from_jstr(line)

            if datum is None:
                continue

            document_count += 1

            average = sampler.datum(datum)

            if average is not None:
                print(JSONify.dumps(average))
                sys.stdout.flush()

            processed_count += 1


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except KeyError as ex:
        print("sample_average: %s" % repr(ex), file=sys.stderr)
        exit(1)

    finally:
        if cmd.verbose:
            print("sample_average: documents: %d processed: %d" % (document_count, processed_count), file=sys.stderr)



"""
Created on 6 Nov 2019

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_analysis
"""

import optparse

from scs_analysis import version


# --------------------------------------------------------------------------------------------------------------------

class CmdSampleError(object):
    """
    unix command line handler
    """

    def __init__(self):
        """
        Constructor
        """
        self.__parser = optparse.OptionParser(usage="%prog { -l | -s } [-p PRECISION] [-v] "
                                                    "REFERENCE_PATH REPORTED_PATH ERROR_PATH", version=version())

        # mode...
        self.__parser.add_option("--linear", "-l", action="store_true", dest="linear", default=False,
                                 help="error is REPORTED - REFERENCE")

        self.__parser.add_option("--scaling", "-s", action="store_true", dest="scaling", default=False,
                                 help="error is REPORTED / REFERENCE")

        # output...
        self.__parser.add_option("--prec", "-p", type="int", action="store", default=3, dest="precision",
                                 help="precision (default 3 decimal places)")

        self.__parser.add_option("--verbose", "-v", action="store_true", dest="verbose", default=False,
                                 help="report narrative to stderr")

        self.__opts, self.__args = self.__parser.parse_args()


    # ----------------------------------------------------------------------------------------------------------------

    def is_valid(self):
        if not self.linear and not self.scaling:
            return False

        if len(self.__args) != 3:
            return False

        return True


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def linear(self):
        return self.__opts.linear


    @property
    def scaling(self):
        return self.__opts.scaling


    @property
    def precision(self):
        return self.__opts.precision


    @property
    def verbose(self):
        return self.__opts.verbose


    @property
    def reference_path(self):
        return self.__args[0] if len(self.__args) > 0 else None


    @property
    def reported_path(self):
        return self.__args[1] if len(self.__args) > 1 else None


    @property
    def error_path(self):
        return self.__args[2] if len(self.__args) > 2 else None


    # ----------------------------------------------------------------------------------------------------------------

    def print_help(self, file):
        self.__parser.print_help(file)


    def __str__(self, *args, **kwargs):
        return "CmdSampleError:{linear:%s, scaling:%s, precision:%s, verbose:%s, " \
               "reference_path:%s, reported_path:%s, error_path:%s}" % \
               (self.linear, self.scaling, self.precision, self.verbose,
                self.reference_path, self.reported_path, self.error_path)
